from edu_function_tools.validators import (
    EduFunctionValidator,
    EduRunnerValidator,
)


class BaseExportDataRunnerValidator(EduRunnerValidator):
    """Базовый класс валидаторов ранеров функций выгрузки данных для интеграции с "Региональная витрина данных"."""

    def validate(self, runnable):
        """Выполнение валидации."""
        super().validate(runnable=runnable)


class BaseExportDataFunctionValidator(EduFunctionValidator):
    """Базовый класс валидаторов функций выгрузки данных для интеграции с "Региональная витрина данных"."""

    def validate(self, runnable):
        """Выполнение валидации."""
        super().validate(runnable=runnable)



from abc import (
    ABCMeta,
    abstractmethod,
)
from copy import (
    deepcopy,
)
from typing import (
    TYPE_CHECKING,
    Any,
    Iterable,
    Optional,
    Union,
)

from django.contrib.contenttypes.models import (
    ContentType,
)
from django.db.models import (
    Q,
)
from django.db.models.constants import (
    LOOKUP_SEP,
)

from educommon.audit_log.utils import (
    get_model_by_table,
)
from educommon.integration_entities.consts import (
    LOG_OPERATION_MAP,
)
from educommon.integration_entities.enums import (
    EntityLogOperation,
)
from educommon.integration_entities.helpers import (
    EntitySaver,
)

from edu_rdm_integration.core.helpers import (
    Graph,
)
from edu_rdm_integration.core.mapping import (
    MODEL_FIELDS_LOG_FILTER,
)
from edu_rdm_integration.core.utils import (
    build_related_model_graph,
)
from edu_rdm_integration.stages.collect_data.functions.base.caches import (
    LogChange,
)
from edu_rdm_integration.stages.collect_data.functions.base.consts import (
    CACHED_OBJECTS_TYPE_MISMATCH_ERROR,
    NOT_SPECIFIED_QUEUE_TO_MODEL_ERROR,
)


if TYPE_CHECKING:
    from edu_function_tools.caches import (
        EduEntityCache,
    )

    from edu_rdm_integration.rdm_models.models import (
        BaseRDMModel,
    )
    from edu_rdm_integration.stages.collect_data.functions.base.caches import (
        IgnoreLogDependency,
    )


class ReformatLogsMixin:
    """Миксин для преобразования логов к удобному для работы виду в кешах помощников функций."""

    def _reformat_logs(self):
        """Производится преобразование логов к удобному для работы виду.

        Предполагается вложенные словари. На первом уровне ключом будет название
        модели, на втором идентификатор записи.
        """
        for log in self.raw_logs:
            model = get_model_by_table(log.table)._meta.label

            if getattr(self, '_log_only_models', None) and (model not in self._log_only_models):
                # Пропускаем, если модель не входит в список отслеживаемых
                continue

            operation = LOG_OPERATION_MAP[log.operation]

            if operation in EntityLogOperation.values:
                fields = log.transformed_data
            else:
                fields = {}

            # Если данных нет, то LogChange не формируем
            if not fields:
                continue

            log_change = LogChange(
                operation=operation,
                fields=fields,
            )

            if not self._filter_log(model, log_change):
                # Если модель не отслеживается, то запись лога не сохраняем
                continue

            if log_change.operation == EntityLogOperation.DELETE:
                self.logs[model][log.object_id] = [
                    log_change,
                ]
            else:
                self.logs[model][log.object_id].append(log_change)

    @staticmethod
    def _filter_log(model: str, log_change: LogChange) -> bool:
        """Производится проверка изменений на отслеживаемые поля."""
        is_filtered = False

        if model in MODEL_FIELDS_LOG_FILTER[log_change.operation]:
            filter_fields = MODEL_FIELDS_LOG_FILTER[log_change.operation][model]
            if filter_fields:
                # Если заданы конкретные поля, которые должны отслеживать
                for field in log_change.fields:
                    if field in filter_fields:
                        # Достаточно, чтобы хотя бы одно поле попало под фильтр
                        is_filtered = True
                        break
            else:
                # Модель отслеживается, но перечень фильтруемых полей не задан,
                # значит фильтруем все поля модели
                is_filtered = True

        return is_filtered


class BaseIgnoreLogMixin:
    """Миксин для исключения логов не подлежащих обработке.

    Позволяет указать зависимость логов от состояния вышестоящих зависимых объектов.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._graph = Graph()
        self._dependency: Optional['IgnoreLogDependency'] = None

    @staticmethod
    def _add_lookup_prefix(q_filter: Q, lookup_path_args: list[str]) -> Q:
        """Функция принимает Q-объект и добавляет префикс к каждому указанному полю."""
        new_q = Q(_negated=q_filter.negated, _connector=q_filter.connector)

        for child in q_filter.children:
            if isinstance(child, Q):
                new_child = BaseIgnoreLogMixin._add_lookup_prefix(child, lookup_path_args)
            else:
                field_name, lookup_value = child

                if isinstance(field_name, str):
                    field_name = LOOKUP_SEP.join((*lookup_path_args, field_name))
                elif isinstance(field_name, tuple):
                    field_name = (LOOKUP_SEP.join((*lookup_path_args, field_name[0])),) + field_name[1:]

                new_child = (field_name, lookup_value)

            new_q.add(new_child, q_filter.connector)

        return new_q

    def _build_filter(
        self,
        lookup_path_args: list[str],
        filter_params: Union[tuple[Q], dict[str, Any]],
        is_valid_lookup: bool = False,
    ) -> Q:
        """Построение фильтра."""
        # Всегда строим OR, так как соответствие любому из параметров фильтра будет однозначно свидетельствовать, что
        # данные являются некорректными и должны быть исключены из последующей обработки
        q_filter = Q(_connector=Q.OR)

        if is_valid_lookup:
            if isinstance(filter_params, dict):
                for field_name, value in filter_params.items():
                    q_filter.add((LOOKUP_SEP.join((*lookup_path_args, field_name)), value), Q.OR)
            else:
                for filter_param in filter_params:
                    if isinstance(filter_param, Q):
                        q_filter |= self._add_lookup_prefix(filter_param, lookup_path_args)
                    else:
                        raise ValueError('Please, check `DependencyFilter.filters`, incorrect value!')

        return q_filter

    def _exclude_logs(self, model_label: str, object_model_ids: Iterable[int]):
        """Исключаем логи из обработки."""
        for object_id in object_model_ids:
            if object_id in self.logs[model_label]:
                del self.logs[model_label][object_id]

    def _ignore_logs(self):
        """Исключение логов из обработки на основании описанных правил."""
        super()._ignore_logs()

        if not self._dependency:
            return

        model_label_separator = '.'

        for app_label, model_name in (model_label.split(model_label_separator) for model_label in self.logs):
            q_filter = Q(_connector=Q.OR)

            log_model = ContentType.objects.get(app_label=app_label.lower(), model=model_name.lower()).model_class()

            for dependency in self._dependency.dependency_filters:
                app_name, model_name = dependency.model_label.lower().split(model_label_separator)
                find_model = ContentType.objects.get(app_label=app_name, model=model_name).model_class()

                self._graph = build_related_model_graph(
                    self._graph, log_model, find_model, ignored_model_fields=self._dependency.ignore_model_fields
                )

                name_edges = self._graph.get_edges_between_vertices(
                    log_model._meta.model_name, find_model._meta.model_name
                )

                q_filter |= self._build_filter(
                    name_edges, dependency.filters, is_valid_lookup=bool(name_edges or find_model == log_model)
                )

            if q_filter:
                ignore_model_ids = log_model.objects.filter(
                    q_filter,
                    id__in=self.logs[log_model._meta.label],
                ).values_list('pk', flat=True)
                self._exclude_logs(log_model._meta.label, ignore_model_ids)


class FilteredSaveEntitiesFunctionMixin(metaclass=ABCMeta):
    """Миксин, реализующий фильтрацию объектов перед сохранением.

    Предназначен для использования в классах, которые занимаются сохранением сущностей, но при этом хотят исключить
    неизменённые объекты из операции записи, чтобы избежать лишних действий или запросов к базе данных.
    Для работы требует реализации абстрактного свойства `_model_to_cache_map` и `_model_to_save_queue_map`, которое
    определяет, какие сущности и с каким кешем нужно обработать.

    Attributes:
        _filtered_operations (tuple[EntityLogOperation]): : Операции, подлежащие фильтрации (по умолчанию только UPDATE)
        _saved_entities_to_cache_map (dict[type[BaseRDMModel], EduEntityCache]): Копия маппинга моделей на кэши,
            используемая для сравнения изменений
        _ignored_fields (dict[EntityLogOperation, tuple[str]]): Словарь полей, которые следует игнорировать при
            сравнении для каждой операции
    """

    _filtered_operations: tuple[EntityLogOperation, ...] = (EntityLogOperation.UPDATE,)
    _saved_entities_to_cache_map: dict[type['BaseRDMModel'], 'EduEntityCache'] = {}
    _ignored_fields = {
        EntityLogOperation.UPDATE: ('modified', 'operation', 'collecting_sub_stage', 'exporting_sub_stage'),
    }

    def _is_object_to_save(self, saved_object: 'BaseRDMModel', ignore_fields: Optional[Iterable[str]] = None) -> bool:
        """Проверяет, должен ли объект быть добавлен в очередь на сохранение.

        Args:
            saved_object ('BaseEntityModel'): Объект, который предполагается сохранить.
            ignore_fields (Optional[Iterable[str]]): Дополнительные поля, которые нужно игнорировать.

        Returns:
            bool: True, если объект был изменён и его нужно сохранить, иначе False.
        """
        save_object_type = type(saved_object)
        cache = self._saved_entities_to_cache_map.get(save_object_type)

        modified = True

        if cache and saved_object.pk:
            original_object = cache.get(id=saved_object.pk)

            if original_object:
                modified = self._is_save_object_modified(saved_object, original_object, ignore_fields)

        return modified

    def _is_save_object_modified(
        self,
        saved_object: 'BaseRDMModel',
        original_object: 'BaseRDMModel',
        ignore_fields: Optional[Iterable[str]] = None,
    ) -> bool:
        """Проверяет, были ли изменения в указанном объекте относительно оригинала.

        Args:
            saved_object ('BaseEntityModel'): Объект, который предполагается сохранить.
            original_object ('BaseEntityModel'): Оригинальное значение из кэша.
            ignore_fields (Optional[Iterable[str]]): Поля, которые следует игнорировать при сравнении.

        Returns:
            bool: True, если объект был изменён, иначе False.
        """
        if not isinstance(saved_object, type(original_object)):
            raise ValueError(CACHED_OBJECTS_TYPE_MISMATCH_ERROR)

        if saved_object.operation == EntityLogOperation.DELETE:
            return True

        if ignore_fields is None:
            ignore_fields = []

        # Сравниваем поля объектов
        for field in original_object._meta.fields:
            field_name = field.name

            # Пропускаем игнорируемые поля
            if field_name in ignore_fields:
                continue

            # Получаем текущее значение поля из измененного объекта
            current_value = getattr(saved_object, field_name)

            # Получаем оригинальное значение поля из кеша
            original_value = getattr(original_object, field_name)

            # Сравниваем значения
            if current_value != original_value:
                return True

        # Если все поля совпадают, объект не изменился
        return False

    def _before_prepare(self, *args, **kwargs):
        """Подготовительный этап перед началом обработки объектов.

        Сохраняет копию текущих сопоставлений объектов и кэшей, чтобы обеспечить чистое состояние для последующего
        сравнения.
        """
        self._saved_entities_to_cache_map = {
            model: deepcopy(cache) for model, cache in self._model_to_cache_map.items()
        }

        super()._before_prepare(*args, **kwargs)

    @property
    @abstractmethod
    def _model_to_cache_map(self) -> dict[type['BaseRDMModel'], 'EduEntityCache']:
        """Абстрактное свойство, возвращающее соответствие между типами моделей и их кэшами.

        Должно быть реализовано в дочернем классе. Используется для получения оригинальных данных объектов
        перед сравнением.

        Returns:
            dict[type[BaseRDMModel], EduEntityCache]: Словарь, где ключ — это тип модели, а значение —
                соответствующий кэш этой модели.
        """

    @property
    @abstractmethod
    def _model_to_save_queue_map(self) -> dict[type['BaseRDMModel'], dict]:
        """Абстрактное свойство, возвращающее соответствие между типами моделей и очередями сохранения.

        Должно быть реализовано в дочернем классе. Используется для определения, в какую очередь будет добавляться
        объект для сохранения.

        Returns:
            dict[type[BaseRDMModel], dict]: Словарь, где ключ — это тип модели, а значение — словарь, представляющий
                очередь сохранения для этой модели.
        """

    def _add_to_save_entities(
        self,
        save_object: 'BaseRDMModel',
        operation: EntityLogOperation,
    ):
        """Добавление в очередь сущности на сохранение."""
        try:
            to_save_queue: dict[EntityLogOperation, dict[int, 'BaseRDMModel']] = self._model_to_save_queue_map[
                type(save_object)
            ]
        except KeyError:
            raise KeyError(NOT_SPECIFIED_QUEUE_TO_MODEL_ERROR.format(save_object._meta.model_name))

        save_object.operation = operation
        save_object.collecting_sub_stage = self._sub_stage

        if operation == EntityLogOperation.CREATE:
            to_save_queue[EntityLogOperation.CREATE][save_object.id] = save_object
        elif operation in {EntityLogOperation.UPDATE, EntityLogOperation.DELETE}:
            if self._is_object_to_save(save_object, self._ignored_fields.get(operation)):
                to_save_queue[EntityLogOperation.UPDATE][save_object.id] = save_object

    def _get_entity_saver(self, model: 'BaseRDMModel') -> EntitySaver:
        """Возвращает экземпляр saver'а для указанной модели.

        Args:
            model (BaseRDMModel): Тип модели, для которой требуется получить saver.

        Returns:
            EntitySaver: Объект, отвечающий за сохранение сущностей указанного типа.
        """
        to_save_queue = self._model_to_save_queue_map[model]
        return EntitySaver(to_save_entities=to_save_queue, model=model)

    def _save_entities(self):
        """Выполняет сохранение всех отфильтрованных и подготовленных объектов.

        Перебирает все модели из `_model_to_save_queue_map` и вызывает соответствующий `EntitySaver`.
        """
        for model in self._model_to_save_queue_map:
            entity_saver = self._get_entity_saver(model)
            entity_saver()



# Generated by Django 3.1.14 on 2024-08-19 14:23

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration', '0009_auto_20240522_1619'),
    ]

    operations = [
        migrations.AddField(
            model_name='transferredentity',
            name='export_enabled',
            field=models.BooleanField(default=True, verbose_name='Включение экспорта для сущности'),
        ),
    ]



import celery
from celery.schedules import (
    crontab,
)
from django.conf import (
    settings,
)

from educommon.async_task.models import (
    AsyncTaskType,
)
from educommon.async_task.tasks import (
    UniquePeriodicAsyncTask,
)

from edu_rdm_integration.core.consts import (
    TASK_QUEUE_NAME,
)
from edu_rdm_integration.stages.collect_data.helpers import (
    set_failed_status_suspended_collecting_data_stages,
)
from edu_rdm_integration.stages.export_data.helpers import (
    set_failed_status_suspended_exporting_data_stages,
)


class CheckSuspendedExportedStagePeriodicTask(UniquePeriodicAsyncTask):
    """Периодическая задача поиска зависших этапов/подэтапов экспорта."""

    queue = TASK_QUEUE_NAME
    routing_key = TASK_QUEUE_NAME
    description = 'Поиск зависших этапов/подэтапов экспорта в "Региональная витрина данных"'
    lock_expire_seconds = settings.RDM_CHECK_SUSPEND_TASK_LOCK_EXPIRE_SECONDS
    task_type = AsyncTaskType.SYSTEM
    run_every = crontab(
        minute=settings.RDM_CHECK_SUSPEND_TASK_MINUTE,
        hour=settings.RDM_CHECK_SUSPEND_TASK_HOUR,
        day_of_week=settings.RDM_CHECK_SUSPEND_TASK_DAY_OF_WEEK,
    )

    def process(self, *args, **kwargs):
        """Выполнение задачи."""
        super().process(*args, **kwargs)

        change_status_collecting_result = set_failed_status_suspended_collecting_data_stages()
        change_status_exporting_result = set_failed_status_suspended_exporting_data_stages()

        task_result = {
            'Прервано сборок': (
                f'Этапов {change_status_collecting_result["change_stage_count"]}'
                f' и подэтапов {change_status_collecting_result["change_sub_stage_count"]}'
            ),
            'Прервано выгрузок': (
                f'Этапов {change_status_exporting_result["change_stage_count"]}'
                f' и подэтапов {change_status_exporting_result["change_sub_stage_count"]}'
            ),
        }

        self.set_progress(values=task_result)


celery_app = celery.app.app_or_default()
celery_app.register_task(CheckSuspendedExportedStagePeriodicTask)



from django.apps import (
    AppConfig,
)


class UploaderLoggerConfig(AppConfig):  # noqa D101
    """Приложение для работы с логами выгрузки данных в РВД."""

    name = 'edu_rdm_integration.stages.upload_data.uploader_log'
    label = 'edu_rdm_integration_uploader_log'
    verbose_name = 'Журнал логов РВД'



from typing import (
    Optional,
)

from m3.db import (
    BaseEnumerate,
)

from edu_rdm_integration.core.consts import (
    FAST_TRANSFER_TASK_QUEUE_NAME,
    LONG_TRANSFER_TASK_QUEUE_NAME,
    TASK_QUEUE_NAME,
)


class EntityLevelQueueTypeEnum(BaseEnumerate):
    """Тип уровня очереди сущности."""

    FAST = 1
    BASE = 2
    LONG = 3

    values = {FAST: 'Быстрая', BASE: 'Основная', LONG: 'Долгая'}

    celery_queue_maps = {
        FAST: FAST_TRANSFER_TASK_QUEUE_NAME,
        BASE: TASK_QUEUE_NAME,
        LONG: LONG_TRANSFER_TASK_QUEUE_NAME,
    }

    @classmethod
    def get_queue_name(cls, level: int) -> Optional[str]:
        """Возвращает очередь."""
        return cls.celery_queue_maps.get(level)



from m3_ext.ui.all_components import (
    ExtButton,
    ExtDisplayField,
)
from objectpack.ui import (
    BaseEditWindow,
    BaseListWindow,
    anchor100,
    model_fields_to_controls,
)

from educommon.utils.ui import (
    append_template_globals,
)

from edu_rdm_integration.pipelines.transfer.models import (
    TransferredEntity,
)


class TransferredEntityListWindow(BaseListWindow):
    """Окно реестра сущностей для сбора и экспорта данных."""

    def _init_components(self):
        """Инициализация компонентов окна."""
        super()._init_components()

        self.export_off_button = ExtButton(text='Отключить экспорт', handler='offExport')
        self.export_on_button = ExtButton(text='Включить экспорт', handler='onExport')

    def _do_layout(self):
        """Размещение компонентов окна на форме."""
        super()._do_layout()

        self.grid.top_bar.items.extend((self.export_off_button, self.export_on_button))

    def set_params(self, params, *args, **kwargs):
        """Настройка окна."""
        super().set_params(params, *args, **kwargs)

        append_template_globals(self, 'ui-js/transferred-entity-list.js')
        self.export_change_action_url = params['pack'].export_change_action.get_absolute_url()
        self.pack = params['pack']


class TransferredEntityEditWindow(BaseEditWindow):
    """Окно редактирования сущностей."""

    def _init_components(self):
        """Инициализация компонентов."""
        super()._init_components()

        self._controls = model_fields_to_controls(
            TransferredEntity,
            self,
            field_list=[
                'queue_level',
            ],
        )
        self.entity_name_field = ExtDisplayField(
            read_only=True,
            label='Сущность',
            name='entity_name',
        )
        self._controls.insert(0, self.entity_name_field)

    def _do_layout(self):
        """Расположение компонентов."""
        super()._do_layout()

        self.form.items.extend(list(map(anchor100, self._controls)))

    def set_params(self, params, *args, **kwargs):
        """Простановка парметров."""
        super().set_params(params, *args, **kwargs)

        obj = params.get('object')
        if obj:
            self.entity_name_field.value = obj.entity_id



import logging
import os
from typing import (
    Union,
)

from django.conf import (
    settings,
)

from educommon import (
    logger,
)

from edu_rdm_integration.core.operations import (
    BaseOperationData,
)
from edu_rdm_integration.core.redis_cache import (
    AbstractCache,
)
from edu_rdm_integration.stages.export_data.consts import (
    TOTAL_ATTACHMENTS_SIZE_KEY,
)
from edu_rdm_integration.stages.upload_data.export_managers import (
    ExportQueueSender,
    WorkerSender,
)
from edu_rdm_integration.stages.upload_data.queues import (
    Queue,
)


class UploadData(BaseOperationData):
    """Класс отправки файлов в витрину."""

    def __init__(
        self,
        data_cache: AbstractCache,
        queue: Queue,
        **kwargs,
    ):
        super().__init__(**kwargs)

        self.data_cache = data_cache
        self.queue = queue

        self._configure_agent_client()
        self.result = {
            'total_file_size': 0,  # Общий размер отправленных файлов
            'queue_is_full': False,  # Признак переполнения очереди
            'uploaded_entities': '',  # Список сущностей, попавших в выгрузку
        }

    @property
    def _log_file_path(self) -> Union[str, bytes]:
        """Путь до лог файла."""
        return os.path.join(settings.MEDIA_ROOT, settings.RDM_UPLOAD_LOG_DIR, 'upload_entity.log')

    def _add_file_handler(self) -> None:
        """Добавляет обработчик логов."""
        self._file_handler = logging.FileHandler(self._log_file_path)

        logging.getLogger('info_logger').addHandler(self._file_handler)
        logging.getLogger('exception_logger').addHandler(self._file_handler)

    # TODO https://jira.bars.group/browse/EDUSCHL-22492. Вынужденная мера, т.к. при запуске команды не производится
    #  проверка готовности конфигов приложений. Нужно переработать механизм конфигурирования клиента загрузчика.
    def _configure_agent_client(self):
        """Конфигурирование клиента загрузчика данных в Витрину."""
        import uploader_client
        from django.core.cache import (
            DEFAULT_CACHE_ALIAS,
            caches,
        )
        from uploader_client.contrib.rdm.interfaces.configurations import (
            RegionalDataMartEmulationUploaderConfig,
            RegionalDataMartUploaderConfig,
        )

        if settings.RDM_UPLOADER_CLIENT_ENABLE_REQUEST_EMULATION:
            uploader_client.set_config(
                RegionalDataMartEmulationUploaderConfig(
                    interface='uploader_client.contrib.rdm.interfaces.rest.OpenAPIInterfaceEmulation',
                    url=settings.RDM_UPLOADER_CLIENT_URL,
                    datamart_name=settings.RDM_UPLOADER_CLIENT_DATAMART_NAME,
                    timeout=1,
                    request_retries=1,
                    file_status=settings.RDM_RESPONSE_FILE_STATUS,
                )
            )
        elif settings.RDM_UPLOADER_CLIENT_USE_PROXY_API:
            uploader_client.set_config(
                RegionalDataMartUploaderConfig(
                    interface='uploader_client.contrib.rdm.interfaces.rest.ProxyAPIInterface',
                    cache=caches[DEFAULT_CACHE_ALIAS],
                    url=settings.RDM_UPLOADER_CLIENT_URL,
                    datamart_name=settings.RDM_UPLOADER_CLIENT_DATAMART_NAME,
                    timeout=settings.RDM_UPLOADER_CLIENT_REQUEST_TIMEOUT,
                    request_retries=settings.RDM_UPLOADER_CLIENT_REQUEST_RETRIES,
                    organization_ogrn=settings.RDM_UPLOADER_CLIENT_ORGANIZATION_OGRN,
                    installation_name=settings.RDM_UPLOADER_CLIENT_INSTALLATION_NAME,
                    installation_id=settings.RDM_UPLOADER_CLIENT_INSTALLATION_ID,
                    username=settings.RDM_UPLOADER_CLIENT_USERNAME,
                    password=settings.RDM_UPLOADER_CLIENT_PASSWORD,
                )
            )
        else:
            uploader_client.set_config(
                RegionalDataMartUploaderConfig(
                    url=settings.RDM_UPLOADER_CLIENT_URL,
                    datamart_name=settings.RDM_UPLOADER_CLIENT_DATAMART_NAME,
                    timeout=settings.RDM_UPLOADER_CLIENT_REQUEST_TIMEOUT,
                    request_retries=settings.RDM_UPLOADER_CLIENT_REQUEST_RETRIES,
                )
            )

    def update_total_queue_size_in_cache(self, received_files_size: int):
        """Обновление размера файлов в кеш."""
        with self.data_cache.lock(f'{TOTAL_ATTACHMENTS_SIZE_KEY}:lock', timeout=300):
            queue_total_file_size = self.data_cache.get(TOTAL_ATTACHMENTS_SIZE_KEY) or 0
            if queue_total_file_size:
                queue_total_file_size -= received_files_size
                if queue_total_file_size > 0:
                    self.data_cache.set(
                        TOTAL_ATTACHMENTS_SIZE_KEY,
                        queue_total_file_size,
                        timeout=settings.RDM_REDIS_CACHE_TIMEOUT_SECONDS,
                    )

    def upload_data(self, *args, **kwargs):
        """Запускает отправку данных в витрину."""
        try:
            exporter = ExportQueueSender(self.data_cache, self.queue, settings.RDM_UPLOAD_DATA_TASK_EXPORT_STAGES)
            exporter.run()

            self.result['queue_is_full'] = exporter.queue_is_full
            self.result['total_file_size'] = exporter.queue_total_file_size

            # Если очередь не переполнена - то отправляем данные в витрину
            if not exporter.queue_is_full:
                sender = WorkerSender(self.queue)
                sender.run()

                if sender.entities:
                    self.result['uploaded_entities'] = ','.join(sender.entities)

                if sender.received_file_size:
                    self.update_total_queue_size_in_cache(sender.received_file_size)

        except Exception as err:
            logger.exception(err)
            raise err
        finally:
            self._remove_file_handler()

        return self.result



from typing import (
    Iterable,
)

from m3_ext.ui import (
    all_components as ext,
)
from m3_ext.ui.icons import (
    Icons,
)
from objectpack.ui import (
    BaseEditWindow,
    BaseWindow,
    ComboBoxWithStore,
)

from educommon.objectpack.ui import (
    BaseListWindow,
)
from educommon.utils.ui import (
    append_template_globals,
)

from edu_rdm_integration.pipelines.transfer.enums import (
    EntityLevelQueueTypeEnum,
)


class CommandProgressListWindow(BaseListWindow):
    """Окно списка команд на сбор/экспорт данных."""

    def set_params(self, params):
        """Устанавливает параметры окна."""
        super().set_params(params)

        self.maximized = True
        append_template_globals(self, 'ui-js/start-task.js')
        append_template_globals(self, 'ui-js/async-task-revoke.js')

        self.start_task_url = params['start_task_url']
        self.revoke_url = params['revoke_url']
        self.queue_select_win_url = params['queue_select_win_url']

    def _init_components(self):
        """Инициализирует компоненты окна."""
        super()._init_components()

        self.start_task_button = ext.ExtButton(
            text='Запустить команду',
            icon_cls=Icons.APPLICATION_GO,
            handler='startTask',
        )
        self.revoke_task_button = ext.ExtButton(
            text='Отменить',
            icon_cls=Icons.CANCEL,
            handler='revokeTask',
        )

    def _do_layout(self):
        """Располагает компоненты окна."""
        super()._do_layout()

        self.grid.top_bar.items.insert(1, self.start_task_button)
        self.grid.top_bar.items.append(self.revoke_task_button)


class BaseCreateCommandWindow(BaseEditWindow):
    """Базовое окно создания команды на сбор/экспорт данных."""

    def _init_components(self):
        """Инициализация компонентов."""
        super()._init_components()

        # Поля, которые нужно добавить на форму:
        self.items_: Iterable = ()

    def _do_layout(self):
        """Расположение компонентов."""
        super()._do_layout()

        self.form.items.extend(self.items_)

    def set_params(self, params):
        """Параметры окна."""
        super().set_params(params)

        self.form.label_width = 150
        self.width = 400
        self.height = 'auto'


class CommandQueueSelectWindow(BaseWindow):
    """Окно выбора очереди для запуска периодических задач команд."""

    def _init_components(self):
        """Инициализация компонентов."""
        super()._init_components()

        self.queue_level_combobox = ComboBoxWithStore(
            label='Очередь выполнения команд',
            name='queue_level',
            data=EntityLevelQueueTypeEnum.get_choices(),
            value=EntityLevelQueueTypeEnum.BASE,
            allow_blank=False,
            editable=False,
            anchor='100%',
        )
        self.close_btn = self.btn_close = ext.ExtButton(
            name='close_btn',
            text='Закрыть',
            handler='function(){Ext.getCmp("%s").close();}' % self.client_id,
        )
        self.start_task_button = ext.ExtButton(
            name='start_task',
            text='Запустить команду',
            handler='function(){ win.fireEvent("closed_ok");}',
        )

    def _do_layout(self):
        """Расположение компонентов."""
        super()._do_layout()

        self.items.append(self.queue_level_combobox)
        self.buttons.extend(
            (
                self.start_task_button,
                self.btn_close,
            )
        )

    def set_params(self, params):
        """Параметры окна."""
        super().set_params(params)

        self.width = 400
        self.height = 'auto'
        self.title = 'Очередь для выполнения команды'



# Generated by Django 3.2.24 on 2025-06-28 14:09

import uuid

import django.db.models.deletion
import django.utils.timezone
from django.db import (
    migrations,
    models,
)

import educommon.django.db.mixins

import edu_rdm_integration.core.utils


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_models', '0001_initial'),
        ('async_task', '0003_alter_runningtask_options'),
        ('edu_rdm_integration_collect_data_stage', '0001_initial'),
        ('rdm_collect_and_export_data', '0003_auto_20250704_0725'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name='EduRdmCollectDataCommandProgress',
                    fields=[
                        (
                            'id',
                            models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
                        ),
                        (
                            'created',
                            models.DateTimeField(default=django.utils.timezone.now, verbose_name='Дата создания'),
                        ),
                        (
                            'logs_period_started_at',
                            models.DateTimeField(verbose_name='Левая граница периода обрабатываемых логов'),
                        ),
                        (
                            'logs_period_ended_at',
                            models.DateTimeField(verbose_name='Правая граница периода обрабатываемых логов'),
                        ),
                        ('generation_id', models.UUIDField(default=uuid.uuid4, verbose_name='Идентификатор генерации')),
                        (
                            'institute_ids',
                            models.JSONField(blank=True, default=list, null=True, verbose_name='id учебного заведения'),
                        ),
                        (
                            'logs_link',
                            models.FileField(
                                max_length=255,
                                upload_to=edu_rdm_integration.core.utils.get_data_command_progress_attachment_path,
                                verbose_name='Ссылка на файл логов',
                            ),
                        ),
                        (
                            'type',
                            models.PositiveSmallIntegerField(
                                choices=[(1, 'Автоматический'), (2, 'Ручной')], verbose_name='Тип команды'
                            ),
                        ),
                        (
                            'model',
                            models.ForeignKey(
                                on_delete=django.db.models.deletion.PROTECT,
                                to='edu_rdm_integration_models.regionaldatamartmodelenum',
                                verbose_name='Модель РВД',
                            ),
                        ),
                        (
                            'stage',
                            models.ForeignKey(
                                blank=True,
                                null=True,
                                on_delete=django.db.models.deletion.SET_NULL,
                                to='edu_rdm_integration_collect_data_stage.collectingexporteddatastage',
                                verbose_name='Этап формирования данных для выгрузки',
                            ),
                        ),
                        (
                            'task',
                            models.ForeignKey(
                                blank=True,
                                null=True,
                                on_delete=django.db.models.deletion.SET_NULL,
                                to='async_task.runningtask',
                                verbose_name='Асинхронная задача',
                            ),
                        ),
                    ],
                    options={
                        'verbose_name': 'Задача по сбору данных',
                        'verbose_name_plural': 'Задачи по сбору данных',
                        'db_table': 'edu_rdm_collecting_data_command_progress',
                        'abstract': False,
                    },
                    bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
                ),
            ],
            database_operations=[],
        ),
    ]



from abc import (
    ABCMeta,
)

from edu_function_tools.functions import (
    EduLazySavingPredefinedQueueFunction,
)
from educommon import (
    logger,
)
from educommon.integration_entities.mixins import (
    EntitiesMixin,
)

from edu_rdm_integration.core.consts import (
    LOGS_DELIMITER,
)
from edu_rdm_integration.stages.collect_data.models import (
    RDMCollectingDataStage,
    RDMCollectingDataSubStage,
    RDMCollectingDataSubStageStatus,
)


class BaseCollectingCalculatedDataFunction(
    EntitiesMixin,
    EduLazySavingPredefinedQueueFunction,
    metaclass=ABCMeta,
):
    """Базовый класс функций сбора данных для интеграции с "Региональная витрина данных"."""

    def __init__(self, *args, stage: RDMCollectingDataStage, **kwargs):
        super().__init__(*args, stage=stage, **kwargs)

        previous_sub_stage = (
            RDMCollectingDataSubStage.objects.filter(
                function_id=self.uuid,
            )
            .order_by('started_at')
            .only('pk')
            .first()
        )

        self._sub_stage = RDMCollectingDataSubStage.objects.create(
            stage=stage,
            function_id=self.uuid,
            previous_id=getattr(previous_sub_stage, 'pk', None),
        )

        self._chunk_index = kwargs.get('chunk_index')

        logger.info(f'{LOGS_DELIMITER * 3}created {repr(self._sub_stage)}')

    def _before_prepare(self, *args, **kwargs):
        """Выполнение действий функций системы."""
        self._sub_stage.status_id = RDMCollectingDataSubStageStatus.IN_PROGRESS.key
        self._sub_stage.save()

        logger.info(f'{LOGS_DELIMITER * 3}change status {repr(self._sub_stage)}')

    def before_run(self, *args, **kwargs):
        """Действия перед запуском."""
        super().before_run(*args, **kwargs)

        logger.info(
            '{delimiter}{force_run}run {runner_name}{log_chunks}..'.format(
                delimiter=LOGS_DELIMITER * 2,
                force_run='force ' if kwargs.get('is_force_run', False) else '',
                runner_name=self.__class__.__name__,
                log_chunks=f' with logs chunk {self._chunk_index}' if self._chunk_index else '',
            )
        )

    def run(self, *args, **kwargs):
        """Выполнение действий функции с дальнейшим сохранением объектов в базу при отсутствии ошибок."""
        super().run(*args, **kwargs)

        if self.result.errors:
            self._sub_stage.status_id = RDMCollectingDataSubStageStatus.FAILED.key
        else:
            self._sub_stage.status_id = RDMCollectingDataSubStageStatus.READY_TO_EXPORT.key

        self._sub_stage.save()

        logger.info(f'{LOGS_DELIMITER * 3}change status {repr(self._sub_stage)}')

        logger.info(f'{LOGS_DELIMITER * 3}{self.__class__.__name__} finished.')



from edu_function_tools.helpers import (
    EduFunctionHelper,
    EduRunnerHelper,
)

from edu_rdm_integration.core.operations import (
    ALL_OPERATIONS,
    UPDATED_OPERATIONS,
)


class BaseCollectingDataRunnerHelper(EduRunnerHelper):
    """Базовый класс помощников ранеров функций сбора данных для интеграции с "Региональная витрина данных"."""


class BaseCollectingDataFunctionHelper(EduFunctionHelper):
    """Базовый класс помощников функций сбора данных для интеграции с "Региональная витрина данных"."""

    def get_filtered_operations(self, with_deleted: bool = False) -> tuple[int]:
        """Возвращает кортеж отфильтрованных операций."""
        return ALL_OPERATIONS if with_deleted else UPDATED_OPERATIONS



# Generated by Django 3.2.24 on 2025-07-04 08:10

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_export_data_stage', '0001_initial'),
        ('edu_rdm_integration_transfer_pipeline', '0001_initial'),
        ('edu_rdm_integration_entities', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='RegionalDataMartEntityEnum',
            new_name='RDMEntityEnum',
        ),
    ]



from django.apps import (
    AppConfig,
)


class RDMCollectDataRegistryAppConfig(AppConfig):
    """Приложение для организации сбора данных моделей РВД из интерфейса."""

    name = 'edu_rdm_integration.stages.collect_data.registry'
    label = 'edu_rdm_integration_collect_data_registry'
    verbose_name = 'Сбор данных моделей РВД из интерфейса'



# Generated by Django 3.2.24 on 2025-10-08 14:43

from django.db import (
    migrations,
)

from edu_rdm_integration.stages.utils import (
    update_foreign_key_constraint,
)


def apply_fk_updates(apps, schema_editor):
    """
    Применяет обновления внешних ключей для конкретных моделей.

    Здесь задаются таблицы и поведение при удалении записей,
    чтобы синхронизировать фактическое состояние БД с логикой моделей.
    """
    Entry = apps.get_model('uploader_client', 'Entry')
    RDMExportingDataSubStage = apps.get_model('edu_rdm_integration_export_data_stage', 'RDMExportingDataSubStage')
    RDMExportingDataSubStageAttachment = apps.get_model(
        'edu_rdm_integration_export_data_stage', 'RDMExportingDataSubStageAttachment'
    )
    RDMExportingDataSubStageUploaderClientLog = apps.get_model(
        'edu_rdm_integration_upload_data_stage', 'RDMExportingDataSubStageUploaderClientLog'
    )
    # Модель: RDMExportingDataSubStageUploaderClientLog
    # Поле sub_stage_id должно иметь ON DELETE CASCADE
    update_foreign_key_constraint(
        table_name=RDMExportingDataSubStageUploaderClientLog._meta.db_table,
        field_name="sub_stage_id",
        target_table=RDMExportingDataSubStage._meta.db_table,
        on_delete="CASCADE",
    )
    # Модель: RDMExportingDataSubStageUploaderClientLog
    # Поле attachment_id должно иметь ON DELETE CASCADE
    update_foreign_key_constraint(
        table_name=RDMExportingDataSubStageUploaderClientLog._meta.db_table,
        field_name="attachment_id",
        target_table=RDMExportingDataSubStageAttachment._meta.db_table,
        on_delete="CASCADE",
    )
    # Модель: RDMExportingDataSubStageUploaderClientLog
    # Поле entry_id должно иметь ON DELETE SET NULL
    update_foreign_key_constraint(
        table_name=RDMExportingDataSubStageUploaderClientLog._meta.db_table,
        field_name="entry_id",
        target_table=Entry._meta.db_table,
        on_delete="SET NULL",
    )

    RDMUploadStatusRequestLog = apps.get_model('edu_rdm_integration_upload_data_stage', 'RDMUploadStatusRequestLog')
    # Модель: RDMUploadStatusRequestLog
    # Поле upload_id должно иметь ON DELETE CASCADE
    update_foreign_key_constraint(
        table_name=RDMUploadStatusRequestLog._meta.db_table,
        field_name="upload_id",
        target_table=RDMExportingDataSubStageUploaderClientLog._meta.db_table,
        on_delete="CASCADE",
    )
    # Модель: RDMUploadStatusRequestLog
    # Поле entry_id должно иметь ON DELETE SET NULL
    update_foreign_key_constraint(
        table_name=RDMUploadStatusRequestLog._meta.db_table,
        field_name="entry_id",
        target_table=Entry._meta.db_table,
        on_delete="SET NULL",
    )


class Migration(migrations.Migration):

    dependencies = [
        ('edu_rdm_integration_upload_data_stage', '0003_auto_20251006_1417'),
    ]

    operations = [
        migrations.RunPython(apply_fk_updates, reverse_code=migrations.RunPython.noop),
    ]



from edu_rdm_integration.stages.upload_data.models import (
    RDMRequestStatus,
)


FAILED_STATUSES = {
    RDMRequestStatus.FAILED_PROCESSING,
    RDMRequestStatus.REQUEST_ID_NOT_FOUND,
    RDMRequestStatus.FLC_ERROR,
}



import asyncio
from abc import (
    ABCMeta,
    abstractmethod,
)
from pathlib import (
    Path,
)
from typing import (
    TYPE_CHECKING,
)

import asyncpg
from django.conf import (
    settings,
)
from django.db import (
    connection,
)

from educommon import (
    logger,
)


if TYPE_CHECKING:
    from asyncpg import (
        Pool,
    )


class BaseServiceOutdatedDataCleaner(metaclass=ABCMeta):
    """Базовый класс уборщика устаревших сервисных данных."""

    model = None

    SELECT_RDM_CHUNK_BOUNDED_SQL = """
        DO $$
        DECLARE
            chunk_size INT := {chunk_size};
            last_id INT := 0;
            first_id INT;
            last_chunk_id INT;
        BEGIN
            DROP TABLE IF EXISTS rdm_chunk_bounds;
            CREATE TEMP TABLE rdm_chunk_bounds (
                chunk_number INT,
                first_id INT,
                last_id INT
            );

            DROP TABLE IF EXISTS tmp_chunk;
            CREATE TEMP TABLE tmp_chunk (id INT) ON COMMIT DROP;

            WHILE TRUE LOOP
                TRUNCATE tmp_chunk;

                INSERT INTO tmp_chunk (id)
                SELECT id
                FROM {table_name}
                WHERE id > last_id
                ORDER BY id
                LIMIT chunk_size;

                IF NOT FOUND THEN
                    EXIT;
                END IF;

                SELECT MIN(id), MAX(id)
                INTO first_id, last_chunk_id
                FROM tmp_chunk;

                INSERT INTO rdm_chunk_bounds (chunk_number, first_id, last_id)
                VALUES (
                    (SELECT COUNT(*) FROM rdm_chunk_bounds) + 1,
                    first_id,
                    last_chunk_id
                );

                last_id := last_chunk_id;
            END LOOP;
        END $$;

        SELECT * FROM rdm_chunk_bounds ORDER BY chunk_number;
    """

    REMOVE_OUTDATED_DATA_SQL = """
        WITH deleted_rows AS (
            DELETE FROM {table_name}
            WHERE id IN (
                WITH tbl AS (
                    SELECT *
                    FROM {table_name}
                    WHERE id >= {first_id}
                        AND id <= {last_id}
                )
                SELECT tbl.id
                FROM tbl
                WHERE {conditions}
            )
            RETURNING id
        )
        SELECT COUNT(*) AS deleted_count FROM deleted_rows;
    """

    def __init__(
        self,
        *args,
        safe: bool = False,
        log_sql: bool = False,
        **kwargs
    ):
        """Инициализация уборщика."""
        self._safe = safe
        self._log_sql = log_sql
        self._deleted_count = 0

        super().__init__(*args, **kwargs)

    @abstractmethod
    def get_merged_conditions(self) -> str:
        """Возвращает условия для удаления устаревших данных."""

    @classmethod
    def get_table_name(cls) -> str:
        """Возвращает имя таблицы в базе данных."""
        if cls.model is None:
            raise NotImplementedError('Необходимо задать атрибут "model"')

        return cls.model._meta.db_table

    async def file_deletion_process(self, file_paths: list[str]):
        """Функция для удаления файлов, связанных с удалёнными устаревшими записями.

        Очистка данных производится в таблицах системных моделей РВД.
        """

    def get_orphan_reference_condition(
        self,
        reference_table: str,
        reference_field: str,
        local_field: str = 'id'
    ) -> str:
        """Условие проверки отсутствия записей в связанной таблице."""
        return f"""
            NOT EXISTS (
                SELECT 1
                FROM {reference_table} ref
                WHERE ref.{reference_field} = tbl.{local_field}
            )
        """

    def get_status_condition(
        self,
        related_table: str,
        related_field: str,
        status_value: str,
        days: int,
        local_field: str = 'id'
    ) -> str:
        """Условие проверки записи с заданным статусом и возрастом."""
        return f"""
            EXISTS (
                SELECT 1
                FROM {related_table} sub
                WHERE sub.{related_field} = tbl.{local_field}
                  AND sub.status_id = '{status_value}'
                  AND sub.ended_at <= NOW() - INTERVAL '{days} days'
            )
        """

    def get_chunk_bounded(self):
        """Возвращает границы чанков для текущей таблицы."""
        get_chunk_bounded_sql = self.SELECT_RDM_CHUNK_BOUNDED_SQL.format(
            table_name=self.get_table_name(),
            chunk_size=settings.RDM_CLEANUP_MODELS_OUTDATED_DATA_CHUNK_SIZE,
        )

        if self._log_sql:
            # Проверка на доступность sqlparse для форматирования
            try:
                import sqlparse
            except ImportError:
                sqlparse = None

            if sqlparse:
                # Форматирование кода
                get_chunk_bounded_sql = sqlparse.format(
                    sql=get_chunk_bounded_sql,
                    reindent=True,
                    strip_comments=True,
                )
            logger.info(
                f'Запрос для получения границ чанков модели {self.get_table_name()}: \n{get_chunk_bounded_sql}\n'
            )

        with connection.cursor() as cursor:
            cursor.execute(get_chunk_bounded_sql)
            result = cursor.fetchall()

        return result

    async def execute_query(self, pool: 'Pool', query: str):
        """Асинхронное выполнение запроса."""
        async with pool.acquire() as conn:
            try:
                if self._safe:
                    logger.info(f'Запрос не будет выполнен, включен безопасный режим!\n')

                    if self._log_sql:
                        logger.info(f'{query}\n')
                else:
                    result = await conn.fetch(query)
                    if not result:
                        return

                    if self._log_sql:
                        logger.info(f'При помощи запроса:\n{query}\n')

                    # Проверяем, что вернул запрос
                    if 'deleted_count' in result[0]:
                        deleted_count = result[0]['deleted_count']
                        self._deleted_count += deleted_count
                        logger.info(f'Было удалено записей: {deleted_count}')
                    else:
                        file_paths = [record['file_path'] for record in result if record.get('file_path')]
                        if file_paths:
                            await self.file_deletion_process(file_paths)
                        deleted_count = len(result)
                        self._deleted_count += deleted_count
                        logger.info(f'Было удалено записей с файлами: {deleted_count}')

            except Exception as e:
                logger.error(f'Ошибка при выполнении {query}\n{e}')

    def prepare_queries(self, chunk_bounded: list[tuple[int, int, int]]) -> list[str]:
        """Формирование списка запросов для удаления устаревших данных."""
        queries = []
        conditions = self.get_merged_conditions()

        for chunk_number, first_id, last_id in chunk_bounded:
            remove_outdated_data_sql = self.REMOVE_OUTDATED_DATA_SQL.format(
                table_name=self.get_table_name(),
                first_id=first_id,
                last_id=last_id,
                conditions=conditions,
            )

            queries.append(remove_outdated_data_sql)

        return queries

    async def execute_queries(self, queries: list[str]) -> None:
        """Асинхронное выполнение запросов."""
        DB_SETTINGS = settings.DATABASES['default']

        pool = await asyncpg.create_pool(
            max_size=settings.RDM_CLEANUP_MODELS_OUTDATED_DATA_POOL_SIZE,
            min_size=settings.RDM_CLEANUP_MODELS_OUTDATED_DATA_POOL_SIZE,
            host=DB_SETTINGS['HOST'],
            port=DB_SETTINGS['PORT'],
            user=DB_SETTINGS['USER'],
            password=DB_SETTINGS['PASSWORD'],
            database=DB_SETTINGS['NAME'],
        )

        tasks = [self.execute_query(pool, query) for query in queries]

        await asyncio.gather(*tasks)

    def run(self):
        """Запуск очистки устаревших данных."""
        chunk_bounded = self.get_chunk_bounded()

        queries = self.prepare_queries(chunk_bounded=chunk_bounded)

        if queries:
            even_loop = asyncio.new_event_loop()
            try:
                even_loop.run_until_complete(self.execute_queries(queries=queries))
            finally:
                even_loop.close()

        logger.info(f'Удалено записей модели {self.model.__name__}: {self._deleted_count}')


class ServiceFileCleaner:
    """Асинхронный сервис для безопасного удаления файлов из MEDIA_ROOT."""

    @staticmethod
    async def file_deletion_process(file_paths: list[str]) -> None:
        """Удаляет указанные файлы, считая пути относительными к MEDIA_ROOT."""
        media_root = Path(settings.MEDIA_ROOT).resolve()

        async def delete_file(path_str: str):
            path = (media_root / path_str).resolve()
            try:
                exists = await asyncio.to_thread(path.exists)
                if exists and await asyncio.to_thread(path.is_file):
                    await asyncio.to_thread(path.unlink)

            except Exception as e:
                logger.warning(f"Не удалось удалить {path}: {e}")

        await asyncio.gather(*(delete_file(path) for path in file_paths))



from django.apps import (
    AppConfig,
)


class RDMExportDataStageAppConfig(AppConfig):
    """Приложение для организации этапа выгрузки собранных данных в файлы."""

    name = 'edu_rdm_integration.stages.export_data'
    label = 'edu_rdm_integration_export_data_stage'
    verbose_name = 'Этап выгрузки данных в файлы'



from dataclasses import (
    dataclass,
)
from datetime import (
    datetime,
)
from typing import (
    Optional,
)

from educommon.integration_entities.entities import (
    BaseEntity,
)
from m3_db_utils.models import (
    ModelEnumValue,
)

from edu_rdm_integration.rdm_entities.mixins import (
    EntityEnumRegisterMixin,
)
from edu_rdm_integration.rdm_models.models import (
    RDMModelEnum,
)


@dataclass
class AddressEntity(EntityEnumRegisterMixin, BaseEntity):
    """Сущность РВД 'Адреса'."""

    # Параметры регистрации в модель-перечисление РВД
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    title = 'Адреса'
    # TODO EDUSCHL-20938 Рефакторинг в рамках задачи
    # main_model_enum = RegionalDataMartModelEnum.ADDRESS
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Имена полей объявляются как константы, чтобы можно было выводить списки полей
    ID = 'id'
    USE_ADDRESS = 'use_address'
    TYPE_ADDRESS = 'type_address'
    TEXT = 'text'
    CITY = 'city'
    STATE = 'state'
    DISTRICT = 'district'
    POSTAL_CODE = 'postal_code'
    COUNTRY = 'country'
    START_DATETIME = 'start_datetime'
    FLAT = 'flat'
    BUILDING = 'building'
    HOUSE = 'house'
    STREET = 'street'
    CREATE_DATETIME = 'create_datetime'
    SEND_DATETIME = 'send_datetime'
    END_DATETIME = 'end_datetime'

    id: str
    use_address: int
    type_address: int
    text: str
    city: str
    state: str
    district: str
    postal_code: str
    country: str
    start_datetime: datetime
    flat: str
    building: str
    house: str
    street: str
    create_datetime: datetime
    send_datetime: datetime
    end_datetime: datetime = None

    @classmethod
    def get_required_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж обязательных полей."""
        return cls.ID, cls.TYPE_ADDRESS, cls.TEXT, cls.CREATE_DATETIME, cls.SEND_DATETIME

    @classmethod
    def get_hashable_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей, которые необходимо деперсонализировать (хэшировать)."""
        return tuple()

    @classmethod
    def get_primary_key_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей первичного ключа."""
        return (cls.ID,)

    @classmethod
    def get_foreign_key_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей внешних ключей."""
        return ()

    @classmethod
    def get_ordered_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей в правильном порядке."""
        return (
            cls.ID,
            cls.USE_ADDRESS,
            cls.TYPE_ADDRESS,
            cls.TEXT,
            cls.CITY,
            cls.DISTRICT,
            cls.STATE,
            cls.POSTAL_CODE,
            cls.COUNTRY,
            cls.START_DATETIME,
            cls.END_DATETIME,
            cls.FLAT,
            cls.BUILDING,
            cls.HOUSE,
            cls.STREET,
            cls.CREATE_DATETIME,
            cls.SEND_DATETIME,
        )

    # TODO EDUSCHL-20938 Удалить в рамках задачи
    @classmethod
    def get_main_model_enum(cls) -> Optional[ModelEnumValue]:
        """Возвращает значение модели перечисление основной модели сущности.

        В классе определяется поле main_model_enum или данный метод.

        !!! Временное решение. Будет удалено в рамках EDUSCHL-20938.
        """
        return RDMModelEnum.ADDRESS


@dataclass
class AddressOrganisationEntity(EntityEnumRegisterMixin, BaseEntity):
    """Сущность РВД 'Адреса'."""

    # Параметры регистрации в модель-перечисление РВД
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    title = 'Адреса организаций'
    # TODO EDUSCHL-20938 Рефакторинг в рамках задачи
    # main_model_enum = RegionalDataMartModelEnum.ADDRESS_ORGANISATION
    # TODO EDUSCHL-20938 Рефакторинг в рамках задачи
    # additional_model_enums = (RegionalDataMartModelEnum.ORGANISATION, )
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Имена полей объявляются как константы, чтобы можно было выводить списки полей
    ADDRESS_ID = 'address_id'
    ORGANISATION_ID = 'organisation_id'
    CREATE_DATETIME = 'create_datetime'
    SEND_DATETIME = 'send_datetime'

    address_id: str
    organisation_id: str
    create_datetime: datetime
    send_datetime: datetime

    @classmethod
    def get_required_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж обязательных полей."""
        return cls.ADDRESS_ID, cls.ORGANISATION_ID, cls.CREATE_DATETIME, cls.SEND_DATETIME

    @classmethod
    def get_hashable_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей, которые необходимо деперсонализировать (хэшировать)."""
        return tuple()

    @classmethod
    def get_primary_key_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей первичного ключа."""
        return (
            cls.ADDRESS_ID,
            cls.ORGANISATION_ID,
        )

    @classmethod
    def get_foreign_key_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей внешних ключей."""
        return ()

    @classmethod
    def get_ordered_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей в правильном порядке."""
        return (
            cls.ADDRESS_ID,
            cls.ORGANISATION_ID,
            cls.CREATE_DATETIME,
            cls.SEND_DATETIME,
        )

    # TODO EDUSCHL-20938 Удалить в рамках задачи
    @classmethod
    def get_main_model_enum(cls) -> Optional[ModelEnumValue]:
        """Возвращает значение модели перечисление основной модели сущности.

        В классе определяется поле main_model_enum или данный метод.

        !!! Временное решение. Будет удалено в рамках EDUSCHL-20938.
        """
        return RDMModelEnum.ADDRESS_ORGANISATION

    # TODO EDUSCHL-20938 Удалить в рамках задачи
    @classmethod
    def get_additional_model_enums(cls) -> Optional[tuple[ModelEnumValue]]:
        """Возвращает кортеж значений модели-перечисления основной модели сущности.

        В классе определяется поле additional_model_enums или данный метод.

        !!! Временное решение. Будет удалено в рамках EDUSCHL-20938.
        """
        return (RDMModelEnum.ORGANISATION,)


@dataclass
class TelecomEntity(EntityEnumRegisterMixin, BaseEntity):
    """Сущность РВД "Контактные данные"."""

    # Параметры регистрации в модель-перечисление РВД
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    title = 'Контактные данные'
    # TODO EDUSCHL-20938 Рефакторинг в рамках задачи
    # main_model_enum = RegionalDataMartModelEnum.TELECOM
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Имена полей объявляются как константы, чтобы можно было выводить списки полей
    ID = 'id'
    TYPE_CONTACT = 'type_contact'
    VALUE_TYPE = 'value_type'
    USE_COD = 'use_cod'
    RANK_CONTACT = 'rank_contact'
    START_DATETIME = 'start_datetime'
    END_DATETIME = 'end_datetime'
    CREATE_DATETIME = 'create_datetime'
    SEND_DATETIME = 'send_datetime'

    id: str
    type_contact: int
    value_type: str
    use_cod: int
    start_datetime: datetime
    create_datetime: datetime
    send_datetime: datetime

    # Данные для полей не собираются
    rank_contact: Optional[int] = None
    end_datetime: datetime = None

    @classmethod
    def get_ordered_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей в правильном порядке."""
        return (
            cls.ID,
            cls.TYPE_CONTACT,
            cls.VALUE_TYPE,
            cls.USE_COD,
            cls.RANK_CONTACT,
            cls.START_DATETIME,
            cls.END_DATETIME,
            cls.CREATE_DATETIME,
            cls.SEND_DATETIME,
        )

    @classmethod
    def get_primary_key_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей первичного ключа."""
        return (cls.ID,)

    @classmethod
    def get_foreign_key_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей внешних ключей."""
        return ()

    @classmethod
    def get_required_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж обязательных полей."""
        return (
            cls.ID,
            cls.CREATE_DATETIME,
            cls.SEND_DATETIME,
        )

    @classmethod
    def get_hashable_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей, которые необходимо деперсонализировать (хэшировать)."""
        return tuple()

    # TODO EDUSCHL-20938 Удалить в рамках задачи
    @classmethod
    def get_main_model_enum(cls) -> Optional[ModelEnumValue]:
        """Возвращает значение модели перечисление основной модели сущности.

        В классе определяется поле main_model_enum или данный метод.

        !!! Временное решение. Будет удалено в рамках EDUSCHL-20938.
        """
        return RDMModelEnum.TELECOM


@dataclass
class OrganisationsEntity(EntityEnumRegisterMixin, BaseEntity):
    """Сущность РВД "Организации"."""

    # Параметры регистрации в модель-перечисление РВД
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    title = 'Организации'
    # TODO EDUSCHL-20938 Рефакторинг в рамках задачи
    # main_model_enum = RegionalDataMartModelEnum.ORGANISATION
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    ID = 'id'
    ROSOBR_ID = 'rosobr_id'
    OGRN = 'ogrn'
    OKPO = 'okpo'
    INN = 'inn'
    KPP = 'kpp'
    ACTIVE = 'active'
    OKPF = 'okpf'
    OKFS = 'okfs'
    OKOGU = 'okogu'
    OKATO = 'okato'
    OKTMO = 'oktmo'
    NAME = 'name'
    PARTOF_ID = 'partof_id'
    EXECUTIVE_NAME = 'executive_name'
    EXECUTIVE_POSITION = 'executive_position'
    MODIFIED = 'modified'
    CREATE_DATETIME = 'create_datetime'
    SEND_DATETIME = 'send_datetime'
    PHONE = 'phone'

    id: str
    rosobr_id: str
    ogrn: str
    okpo: str
    inn: str
    kpp: str
    active: bool
    okpf: str
    okfs: str
    okogu: str
    okato: str
    oktmo: str
    name: str
    partof_id: str
    executive_name: str
    executive_position: str
    modified: datetime
    create_datetime: datetime
    send_datetime: datetime
    phone: str

    @classmethod
    def get_ordered_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей в правильном порядке."""
        return (
            cls.ID,
            cls.ROSOBR_ID,
            cls.OGRN,
            cls.OKPO,
            cls.INN,
            cls.KPP,
            cls.ACTIVE,
            cls.OKPF,
            cls.OKFS,
            cls.OKOGU,
            cls.OKATO,
            cls.OKTMO,
            cls.NAME,
            cls.PARTOF_ID,
            cls.EXECUTIVE_NAME,
            cls.EXECUTIVE_POSITION,
            cls.PHONE,
            cls.CREATE_DATETIME,
            cls.SEND_DATETIME,
        )

    @classmethod
    def get_required_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж обязательных полей."""
        return (
            cls.ID,
            cls.ROSOBR_ID,
            cls.OGRN,
            cls.INN,
            cls.KPP,
            cls.OKPF,
            cls.OKFS,
            cls.OKOGU,
            cls.OKATO,
            cls.OKTMO,
            cls.NAME,
            cls.EXECUTIVE_NAME,
            cls.EXECUTIVE_POSITION,
            cls.PHONE,
            cls.CREATE_DATETIME,
            cls.SEND_DATETIME,
        )

    @classmethod
    def get_primary_key_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей первичного ключа."""
        return (cls.ID,)

    @classmethod
    def get_foreign_key_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей внешних ключей."""
        return (cls.PARTOF_ID,)

    @classmethod
    def get_hashable_fields(cls) -> tuple[str, ...]:
        """Возвращает кортеж полей, которые необходимо деперсонализировать (хэшировать)."""
        return ()

    # TODO EDUSCHL-20938 Удалить в рамках задачи
    @classmethod
    def get_main_model_enum(cls) -> Optional[ModelEnumValue]:
        """Возвращает значение модели перечисление основной модели сущности.

        В классе определяется поле main_model_enum или данный метод.

        !!! Временное решение. Будет удалено в рамках EDUSCHL-20938.
        """
        return RDMModelEnum.ORGANISATION



from edu_rdm_integration.stages.collect_data.functions.base.helpers import (
    BaseCollectingDataFunctionHelper,
    BaseCollectingDataRunnerHelper,
)
from edu_rdm_integration.stages.collect_data.functions.non_calculated.base.caches import (
    BaseCollectingExportedDataFunctionCacheStorage,
    BaseCollectingExportedDataRunnerCacheStorage,
)


class BaseCollectingExportedDataRunnerHelper(BaseCollectingDataRunnerHelper):
    """Базовый класс помощников ранеров функций сбора данных для интеграции с "Региональная витрина данных"."""

    def _prepare_cache_class(self) -> type[BaseCollectingExportedDataRunnerCacheStorage]:
        """Возвращает класс кеша помощника ранера."""
        return BaseCollectingExportedDataRunnerCacheStorage


class BaseCollectingExportedDataFunctionHelper(BaseCollectingDataFunctionHelper):
    """Базовый класс помощников функций сбора данных для интеграции с "Региональная витрина данных"."""

    def _prepare_cache_class(self) -> type[BaseCollectingExportedDataFunctionCacheStorage]:
        """Возвращает класс кеша помощника функции."""
        return BaseCollectingExportedDataFunctionCacheStorage



from m3_ext.ui.fields import (
    ExtTextArea,
)
from objectpack.ui import (
    BaseEditWindow,
)

from educommon.utils.ui import (
    switch_window_in_read_only_mode,
)


class UploaderLogInfoWindow(BaseEditWindow):
    """Окно просмотра лога Загрузчика данных в витрину."""

    def _init_components(self):
        """Инициализация компонентов окна."""
        super(UploaderLogInfoWindow, self)._init_components()
        self.field__request = ExtTextArea(
            label='Запрос',
            name='request',
            anchor='100%',
            height=160,
        )
        self.field__response = ExtTextArea(
            label='Ответ',
            name='response',
            anchor='100%',
            height=160,
        )
        self.field__error = ExtTextArea(
            label='Ошибка',
            name='request_error',
            anchor='100%',
            height=80,
        )
        self.field__attachment = ExtTextArea(
            label='Вложения',
            name='attachment_file',
            anchor='100%',
            height=40,
        )

    def _do_layout(self):
        """Размещение компонентов окна на форме."""
        super(UploaderLogInfoWindow, self)._do_layout()

        self.form.items.extend(
            (
                self.field__request,
                self.field__response,
                self.field__error,
                self.field__attachment,
            )
        )

    def set_params(self, params):
        """Настройка окна."""
        super(UploaderLogInfoWindow, self).set_params(params)

        self.height = 'auto'
        self.width = 700

        switch_window_in_read_only_mode(self)
        self.make_read_only()



from functools import (
    partial,
)

from django.conf import (
    settings,
)
from django.db.models import (
    F,
    Func,
    OuterRef,
    Q,
    Subquery,
)
from django.db.transaction import (
    atomic,
)
from m3.actions.exceptions import (
    ApplicationLogicException,
)

from educommon.async_task.actions import (
    RevokeAsyncTaskAction,
)
from educommon.async_task.models import (
    AsyncTaskStatus,
)
from educommon.utils.conversion import (
    int_or_none,
)
from educommon.utils.ui import (
    ChoicesFilter,
    DatetimeFilterCreator,
)

from edu_rdm_integration.core.enums import (
    CommandType,
)
from edu_rdm_integration.core.helpers import (
    make_download_link,
)
from edu_rdm_integration.core.registry.actions import (
    BaseCommandProgressPack,
    BaseStartTaskAction,
)
from edu_rdm_integration.rdm_models.models import (
    RDMModelEnum,
)
from edu_rdm_integration.stages.collect_data.generators import (
    FirstCollectModelsDataCommandsGenerator,
)
from edu_rdm_integration.stages.collect_data.models import (
    RDMCollectingDataCommandProgress,
    RDMCollectingDataStageStatus,
    RDMCollectingDataSubStage,
    RDMCollectingDataSubStageStatus,
)
from edu_rdm_integration.stages.collect_data.registry.ui import (
    CreateCollectCommandWindow,
    DetailCollectCommandWindow,
)


class BaseCollectingDataProgressPack(BaseCommandProgressPack):
    """Базовый пак команд сбора данных моделей РВД."""

    title = 'Сбор данных моделей РВД'
    model = RDMCollectingDataCommandProgress

    add_window = CreateCollectCommandWindow
    edit_window = DetailCollectCommandWindow

    need_check_permission = True

    select_related = ['task', 'task__status']

    list_sort_order = ('-created', 'model__order_number', 'generation_id')
    date_filter = partial(DatetimeFilterCreator, model)

    columns = [
        {
            'data_index': 'model.pk',
            'header': 'Модель',
            'sortable': True,
            'filter': ChoicesFilter(
                choices=partial(RDMModelEnum.get_choices),
                parser=str,
                lookup=lambda key: Q(model=key) if key else Q(),
            ),
        },
        {
            'data_index': 'task.status.title',
            'header': 'Статус асинхронной задачи',
            'sortable': True,
            'filter': ChoicesFilter(
                choices=partial(AsyncTaskStatus.get_choices),
                parser=str,
                lookup='task__status_id',
            ),
        },
        {
            'data_index': 'type',
            'header': 'Тип команды',
            'filter': ChoicesFilter(
                choices=CommandType.get_choices(),
                parser=int,
                lookup='type',
            ),
            'width': 60,
        },
        {
            'data_index': 'ready_to_export_sub_stages',
            'header': 'Подэтапов выполнено',
            'width': 50,
        },
        {
            'data_index': 'stage.status.key',
            'header': 'Статус сбора',
            'sortable': True,
            'filter': ChoicesFilter(
                choices=partial(RDMCollectingDataStageStatus.get_choices),
                parser=str,
                lookup=lambda key: Q(stage__status=key) if key else Q(),
            ),
            'width': 50,
        },
        {
            'data_index': 'stage.started_at',
            'header': 'Время начала сбора',
            'sortable': True,
            'filter': date_filter('stage__started_at').filter,
        },
        {
            'data_index': 'log_url',
            'header': 'Ссылка на логи',
            'width': 60,
        },
        {
            'data_index': 'logs_period_started_at',
            'header': 'Начало периода',
            'sortable': True,
            'filter': date_filter('logs_period_started_at').filter,
        },
        {
            'data_index': 'logs_period_ended_at',
            'header': 'Конец периода',
            'sortable': True,
            'filter': date_filter('logs_period_ended_at').filter,
        },
        {
            'data_index': 'generation_id',
            'header': 'ID генерации',
            'sortable': True,
        },
        {
            'data_index': 'created',
            'header': 'Дата создания',
            'sortable': True,
        },
    ]

    _start_task_action_cls: BaseStartTaskAction
    _revoke_task_action_cls: RevokeAsyncTaskAction

    def __init__(self):
        super().__init__()
        self.start_task_action = self._start_task_action_cls()
        self.revoke_task_action = self._revoke_task_action_cls()

        self.actions.extend(
            (
                self.start_task_action,
                self.revoke_task_action,
            )
        )

    def get_list_window_params(self, params, request, context):
        """Получает параметры окна списка."""
        params = super().get_list_window_params(params, request, context)

        params['start_task_url'] = self.start_task_action.get_absolute_url()
        params['revoke_url'] = self.revoke_task_action.get_absolute_url()

        return params

    def declare_context(self, action):
        """Объявление контекста."""
        context = super().declare_context(action)

        if action is self.save_action:
            context['logs_period_started_at'] = context['logs_period_ended_at'] = {'type': 'datetime'}
            context['split_by_quantity'] = context['batch_size'] = {'type': 'int_or_none', 'default': None}
            context['institute_count'] = {'type': 'int'}
            context['split_by'] = context['split_mode'] = {'type': 'str', 'default': None}
            context['by_institutes'] = {'type': 'boolean', 'default': False}
            context['institute_ids'] = {'type': 'int_list'}
        elif action is self.start_task_action:
            context['commands'] = {'type': 'int_list'}
            context['queue_level'] = {'type': int, 'default': None}
        elif action is self.revoke_task_action:
            context['async_task_ids'] = {'type': 'str', 'default': ''}

        return context

    def get_rows_query(self, request, context):
        """Возвращает выборку из БД для получения списка данных."""
        query = super().get_rows_query(request, context)

        # Необходимо также рассчитать прогресс сбора:
        query = query.annotate(
            ready_to_export_sub_stages=Subquery(
                RDMCollectingDataSubStage.objects.filter(
                    stage_id=OuterRef('stage_id'),
                    status_id=RDMCollectingDataSubStageStatus.READY_TO_EXPORT.key,
                )
                .annotate(ready_to_export_sub_stages=Func(F('id'), function='Count'))
                .values('ready_to_export_sub_stages')
            )
        )

        return query

    def prepare_row(self, obj, request, context):
        """Подготовка данных для отображения в реестре."""
        obj.log_url = make_download_link(obj.logs_link)

        return obj

    def _get_actual_institute_ids(self):
        """Возвращает кортеж из идентификаторов организаций, данные по которым можно собрать."""
        raise NotImplementedError

    @atomic
    def save_row(self, obj, create_new, request, context, *args, **kwargs):
        """Сохраняет объект.

        Переопределено, т.к. на основе полученных параметров от клиента,
        необходимо сформировать команды на сбор и их сохранить в модели.
        """
        batch_size = int_or_none(context.batch_size)
        if not context.split_by and not batch_size:
            raise ApplicationLogicException('Поле "Размер чанка" обязательно к заполнению')

        split_by_quantity = int_or_none(context.split_by_quantity)
        if context.split_by and not split_by_quantity:
            raise ApplicationLogicException('Поле "Размер подпериода" обязательно к заполнению')

        commands_to_save = FirstCollectModelsDataCommandsGenerator(
            models=[obj.model_id],
            split_by=context.split_by,
            split_mode=context.split_mode,
            split_by_quantity=context.split_by_quantity,
            logs_period_started_at=context.logs_period_started_at,
            logs_period_ended_at=context.logs_period_ended_at,
            batch_size=context.batch_size,
        ).generate_with_split(
            by_institutes=context.by_institutes,
            institute_ids=context.institute_ids,
            institute_count=context.institute_count,
            actual_institute_ids=self._get_actual_institute_ids(),
        )

        if not commands_to_save:
            raise ApplicationLogicException(
                f'Недостаточно данных для обработки модели "{obj.model_id}" '
                f'в указанный период с {context.logs_period_started_at} по {context.logs_period_ended_at}. '
                f'Проверьте наличие данных в указанном временном диапазоне.'
            )

        objs = [
            self.model(
                model_id=obj.model_id,
                logs_period_started_at=command['period_started_at'],
                logs_period_ended_at=command['period_ended_at'],
                generation_id=command['generation_id'],
                institute_ids=command['institute_ids'],
                type=CommandType.MANUAL,
            )
            for command in commands_to_save
        ]
        self.model.objects.bulk_create(objs, batch_size=settings.RDM_COLLECT_PROGRESS_BATCH_SIZE)



from django.conf import (
    settings,
)
from django.core.management import (
    BaseCommand,
)

from edu_rdm_integration.rdm_models.models import (
    RDMModelEnum,
)
from edu_rdm_integration.stages.service.model_outdated_data.managers import (
    ModelOutdatedDataCleanerManager,
)
from edu_rdm_integration.stages.service.service_outdated_data.managers import (
    ServiceOutdatedDataCleanerManager,
)


class Command(BaseCommand):
    """Ночная команда для очистки устаревших данных РВД."""

    nightly_script = True

    help = 'Ночная команда для очистки устаревших данных РВД.'

    def add_arguments(self, parser):
        """Добавляет аргументы командной строки."""
        models = ', '.join(f'{key} - {value.title}' for key, value in RDMModelEnum.get_enum_data().items())
        models_help_text = (
            f'Значением параметра является перечисление моделей РВД, для которых должна быть произведена зачистка '
            f'устаревших данных. '
            f'Перечисление моделей:\n{models}. Если модели не указываются, то зачистка устаревших данных будет '
            f'производиться для всех моделей. Модели перечисляются через запятую без пробелов.'
        )
        parser.add_argument(
            '--models',
            action='store',
            dest='models',
            type=lambda ml: [m.strip().upper() for m in ml.split(',')] if ml else None,
            help=models_help_text,
        )

        # Аргумент для сервисных стадий
        stages_help_text = (
            'Значением параметра является перечисление стадий сервисных данных, '
            'для которых должна быть произведена зачистка устаревших данных. '
            'Перечисление этапов: collect, export, upload. '
            'Если стадии не указываются, то зачистка будет произведена для всех стадий. '
            'Стадии перечисляются через запятую без пробелов.'
        )
        parser.add_argument(
            '--stages',
            action='store',
            dest='stages',
            type=lambda st: [s.strip().lower() for s in st.split(',')] if st else None,
            help=stages_help_text,
        )

        parser.add_argument(
            '--safe',
            action='store_true',
            dest='safe',
            default=False,
            help='Запускать команду в безопасном режиме (без удаления данных, только логирование).',
        )

        parser.add_argument(
            '--log-sql',
            action='store_true',
            dest='log_sql',
            default=False,
            help='Включить логирование SQL-запросов, выполняемых во время работы команды.',
        )

    def _cleanup_model_outdated_data(self, options):
        """Очистка устаревших данных моделей РВД."""
        model_data_cleaner_manager = ModelOutdatedDataCleanerManager(
            models=options['models'],
            safe=options['safe'],
            log_sql=options['log_sql'],
        )
        model_data_cleaner_manager.run()

    def _cleanup_service_outdated_data(self, options):
        """Очистка устаревших данных сервисных моделей РВД."""
        service_data_cleaner_manager = ServiceOutdatedDataCleanerManager(
            stages=options['stages'],
            safe=options['safe'],
            log_sql=options['log_sql'],
        )
        service_data_cleaner_manager.run()

    def handle(self, *args, **options):
        """Запуск очистки устаревших данных РВД."""
        if settings.RDM_ENABLE_CLEANUP_MODELS_OUTDATED_DATA:
            self._cleanup_model_outdated_data(options)

        if settings.RDM_ENABLE_CLEANUP_SERVICE_OUTDATED_DATA:
            self._cleanup_service_outdated_data(options)



from collections import (
    defaultdict,
)
from typing import (
    Optional,
    Type,
)

from django.db.models import (
    CASCADE,
    DateTimeField,
    ForeignKey,
    SmallIntegerField,
)
from m3.db import (
    BaseObjectModel,
)

from educommon.django.db.mixins import (
    ReprStrPreModelMixin,
)
from educommon.integration_entities.enums import (
    EntityLogOperation,
)
from educommon.utils.seqtools import (
    topological_sort,
)
from m3_db_utils.models import (
    FictiveForeignKeyField,
    ModelEnumValue,
    TitledModelEnum,
)


class BaseRDMModel(ReprStrPreModelMixin, BaseObjectModel):
    """Базовая модель РВД."""

    created = DateTimeField(
        verbose_name='Дата создания',
        auto_now_add=True,
        null=True,
        blank=True,
        db_index=True,
    )
    modified = DateTimeField(
        verbose_name='Дата изменения',
        auto_now=True,
        null=True,
        blank=True,
        db_index=True,
    )

    @property
    def attrs_for_repr_str(self):
        """Список атрибутов для отображения экземпляра модели."""
        return ['created', 'modified']

    class Meta:
        abstract = True


class BaseAdditionalRDMModel(BaseRDMModel):
    """Абстрактная вспомогательная модель РВД.

    Является базовым классом для моделей РВД, которые не являются основными для сущностей РВД. Для таких моделей
    производится сбор данных.
    """

    collecting_sub_stage = ForeignKey(
        verbose_name='Подэтап сбора данных',
        to='edu_rdm_integration_collect_data_stage.RDMCollectingDataSubStage',
        on_delete=CASCADE,
    )
    operation = SmallIntegerField(
        verbose_name='Действие',
        choices=EntityLogOperation.get_choices(),
    )

    class Meta:
        abstract = True


class BaseMainRDMModel(BaseAdditionalRDMModel):
    """Абстрактная основная модель РВД.

    Является базовым классом для моделей РВД, которые являются основными для сущностей РВД. Для таких моделей
    производится сбор и выгрузка данных.
    """

    exporting_sub_stage = ForeignKey(
        verbose_name='Подэтап выгрузки данных',
        to='edu_rdm_integration_export_data_stage.RDMExportingDataSubStage',
        blank=True,
        null=True,
        on_delete=CASCADE,
    )

    @property
    def attrs_for_repr_str(self):
        """Список атрибутов для отображения экземпляра модели."""
        return ['collecting_sub_stage', 'exporting_sub_stage', 'operation', 'created', 'modified']

    class Meta:
        abstract = True


class RDMModelEnum(TitledModelEnum):
    """Модель-перечисление моделей "Региональная витрина данных"."""

    is_strict_order_number = True
    """Флаг, указывающий на уникальность порядкового номера элементов модели-перечисления."""

    class Meta:
        db_table = 'rdm_model'
        extensible = True
        verbose_name = 'Модель-перечисление моделей "Региональной витрины данных"'
        verbose_name_plural = 'Модели-перечисления моделей "Региональной витрины данных"'

    @classmethod
    def get_choices(cls) -> list[tuple[str, str]]:
        """Возвращает список кортежей из ключей и ключей перечисления моделей."""
        return [(key, key) for key in sorted(cls.get_model_enum_keys())]

    @classmethod
    def _get_model_relations(cls, model: Type['BaseRDMModel']) -> dict[str, str]:
        """Получение списка связей модели РВД."""
        model_label_enum_data = cls.get_model_label_enum_data()
        model_relations = {}

        for field in model._meta.concrete_fields:
            if isinstance(field, FictiveForeignKeyField):
                model_label = field.to
            elif isinstance(field, ForeignKey):
                model_label = field.related_model._meta.label
            else:
                continue

            if model_label in model_label_enum_data:
                model_key = model_label_enum_data[model_label].key
                model_relations[field.attname] = model_key

        return model_relations

    @classmethod
    def _get_models_relations(cls) -> dict[str, dict[str, str]]:
        """Получение списка связей всех зарегистрированных моделей РВД."""
        model_label_enum_data = cls.get_model_label_enum_data()
        all_models = [model_enum_value.model for model_enum_value in model_label_enum_data.values()]

        models_relations = {}
        for model in all_models:
            model_key = model_label_enum_data[model._meta.label].key
            model_relations = cls._get_model_relations(model=model)

            if model_relations:
                models_relations[model_key] = model_relations

        return models_relations

    @classmethod
    def _sort_models_dependencies(cls, models_dependencies: list[tuple[str, str]]) -> list[str]:
        """Сортировка моделей РВД по зависимости друг от друга."""
        sorted_dependencies_models = topological_sort(models_dependencies)

        ordered_models = [*sorted_dependencies_models.cyclic, *reversed(sorted_dependencies_models.sorted)]

        return ordered_models

    @classmethod
    def _get_ordered_models(cls) -> list[str]:
        """Получение списка моделей РВД в порядке их зависимости.

        Есть возможность указать дополнительные модели, для которых совместно с зарегистрированными моделями РВД будет
        рассчитываться порядок.

        Returns:
            list[str]: список моделей РВД в порядке их зависимости
        """
        enum_keys = cls.get_model_enum_keys()
        models_relations = cls._get_models_relations()

        models_dependencies = []
        for model_key, model_relations in models_relations.items():
            for _, related_model_key in model_relations.items():
                models_dependencies.append((model_key, related_model_key))

        ordered_models = cls._sort_models_dependencies(models_dependencies=models_dependencies)

        without_dependencies = set(enum_keys).difference(set(ordered_models))

        ordered_models = [*without_dependencies, *ordered_models]

        return ordered_models

    @classmethod
    def _recalculate_order_numbers(cls, *args, **kwargs):
        """Перерасчет порядковых номеров элементов модели-перечисления.

        При добавлении новой модели РВД в модель-перечисление производится перерасчет порядковых номеров уже
        добавленных элементов. Порядок моделей РВД выстраивается по зависимости друг от друга. Зависимость определяется
        внешними ключами (ForeignKey) и фиктивными внешними ключами (FictiveForeignKey). Сначала идут модели не имеющие
        зависимостей, затем модели, которые зависят от других моделей. Сортировка моделей РВД происходит по алгоритму
        топологической сортировки.
        """
        ordered_model_keys = cls._get_ordered_models()

        for model_enum_value in cls._get_enum_data().values():
            if model_enum_value.is_manual_order_number:
                try:
                    manual_index_model_key = ordered_model_keys[model_enum_value.order_number - 1]
                except IndexError:
                    continue

                if manual_index_model_key != model_enum_value.key and cls.is_strict_order_number:
                    raise ValueError(
                        f'Order number "{model_enum_value.order_number}" is already in use in the "{cls.__name__}". '
                        f'Please choose a different one.'
                    )
            else:
                model_enum_value.order_number = ordered_model_keys.index(model_enum_value.key) + 1

    @classmethod
    def _calculate_tmp_order_number(cls) -> int:
        """Вычисление временного порядкового номера элемента модели-перечисления."""
        order_number_enum_data = cls._get_order_number_enum_data()

        order_numbers = order_number_enum_data.keys()
        tmp_order_number = max(order_numbers) + 1 if order_number_enum_data else 1

        return tmp_order_number

    @classmethod
    def _validate_manual_order_number(cls, order_number: int, model: Type['BaseRDMModel']):
        """Валидация пользовательского порядкового номера."""
        ordered_models = cls._get_ordered_models()

        try:
            manual_index_model = ordered_models[order_number - 1]
        except IndexError:
            return

        if manual_index_model != model._meta.label and cls.is_strict_order_number:
            raise ValueError(
                f'Order number "{order_number}" is already in use in the "{cls.__name__}". '
                f'Please choose a different one.'
            )

    @classmethod
    def _update_reverse_relations(cls):
        """Обновление обратных связей моделей РВД.

        У значения модели-перечисления
        """
        enum_data = cls._get_enum_data()
        # Обнуление обратных связей всех моделей
        for model_enum_value in enum_data.values():
            model_enum_value.reverse_relations = defaultdict(list)

        models_relations = cls._get_models_relations()

        for model_key, model_relations in models_relations.items():
            for field_name, related_model_key in model_relations.items():
                enum_data[related_model_key].reverse_relations[model_key].append(field_name)

    @classmethod
    def get_model_label_enum_data(cls) -> dict[str, Type['ModelEnumValue']]:
        """Получение словаря значений модели-перечисления с лейблом модели в качестве значения."""
        enum_data = cls._get_enum_data()

        model_label_enum_data = {model_value.model._meta.label: model_value for model_value in enum_data.values()}

        return model_label_enum_data

    @classmethod
    def extend(
        cls,
        key,
        model: Type['BaseRDMModel'] = None,
        title: str = '',
        creating_trigger_models: tuple = (),
        loggable_models: tuple = (),
        order_number: Optional[int] = None,
        *args,
        **kwargs,
    ):
        """Метод расширения модели-перечисления, например из плагина.

        Необходимо, чтобы сама модель-перечисление была расширяемой. Для этого необходимо, чтобы был установлен
        extensible = True в Meta.

        Args:
            key: ключ элемента перечисления, указывается заглавными буквами с разделителем нижнее подчеркивание
            title: название элемента перечисления
            model: модель, регистрируемая в модели-перечислении
            creating_trigger_models: модели продукта, которые инициируют создание записей модели РВД
            loggable_models: модели продукта, отслеживаемые в логах
            order_number: порядковый номер значения модели перечисления используемый при сортировке
            args: порядковые аргументы для модели-перечисления
            kwargs: именованные аргументы для модели-перечисления
        """
        if key.upper() in cls.get_model_enum_keys():
            raise ValueError('Model enum value with key "{key}" already exists.')

        if model is None:
            raise ValueError(f'Trying extend model "{cls.__name__}". Argument "model" is required.')

        is_manual_order_number = order_number is not None

        if order_number is None:
            order_number = cls._calculate_tmp_order_number()
        else:
            cls._validate_manual_order_number(order_number=order_number, model=model)

        model_enum_value = ModelEnumValue(
            key=key,
            model=model,
            title=title,
            creating_trigger_models=creating_trigger_models,
            loggable_models=loggable_models,
            order_number=order_number,
            is_manual_order_number=is_manual_order_number,
            reverse_relations=defaultdict(list),
            **kwargs,
        )
        setattr(cls, key, model_enum_value)

        cls._recalculate_order_numbers()
        cls._update_reverse_relations()



# Generated by Django 3.2.24 on 2025-06-28 14:08

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('edu_rdm_integration', '0017_delete_uploaddatacommand'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name='RegionalDataMartModelEnum',
                    fields=[
                        ('title', models.TextField(verbose_name='расшифровка значения')),
                        (
                            'key',
                            models.CharField(
                                db_index=True, max_length=512, primary_key=True, serialize=False, verbose_name='ключ'
                            ),
                        ),
                        ('order_number', models.PositiveIntegerField(default=100000, verbose_name='Порядковый номер')),
                    ],
                    options={
                        'verbose_name': 'Модель-перечисление моделей "Региональной витрины данных"',
                        'verbose_name_plural': 'Модели-перечисления моделей "Региональной витрины данных"',
                        'db_table': 'rdm_model',
                    },
                ),
            ],
            database_operations=[],
        ),
    ]



from django.apps import (
    AppConfig,
)


class RDMServiceStageAppConfig(AppConfig):
    """Приложение для организации этапа сервисного обслуживания интеграции с РВД."""

    name = 'edu_rdm_integration.stages.service'
    label = 'edu_rdm_integration_service_stage'
    verbose_name = 'Этап сервисного обслуживания'

    def ready(self):
        """Готовность приложения."""
        if self.name == 'edu_rdm_integration.stages.service':
            # ruff: noqa: F401
            from . import (
                tasks,
            )



import heapq
import os
from abc import (
    ABC,
    abstractmethod,
)
from typing import (
    TYPE_CHECKING,
    Dict,
    Iterable,
    Optional,
    Union,
)

from django.conf import (
    settings,
)
from django.utils.html import (
    format_html,
)
from django.utils.safestring import (
    mark_safe,
)
from redis import (
    Redis,
    ResponseError,
)

from educommon.async_task.exceptions import (
    TaskUniqueException,
)
from educommon.async_task.tasks import (
    AsyncTask,
)

from edu_rdm_integration.core.consts import (
    TASK_QUEUE_NAME,
)
from edu_rdm_integration.pipelines.transfer.enums import (
    EntityLevelQueueTypeEnum,
)
from edu_rdm_integration.stages.collect_data.models import (
    RDMCollectingDataCommandProgress,
)
from edu_rdm_integration.stages.export_data.models import (
    RDMExportingDataCommandProgress,
)


if TYPE_CHECKING:
    from django.db.models.query import (
        QuerySet,
    )


class BaseTaskProgressUpdater(ABC):
    """Базовый класс, который обновляет данные в таблицах, хранящих команды сбора/экспорта."""

    @property
    @abstractmethod
    def update_model(self):
        """Основная модель для обновления.

        Необходимо задать в дочернем классе.
        """

    @property
    @abstractmethod
    def async_model(self):
        """Модель асинхронных задач.

        Необходимо задать в дочернем классе.
        """

    def set_async_task(self, commands_to_update: Dict[RDMCollectingDataCommandProgress, str]) -> None:
        """Устанавливает ссылку на асинхронную задачу."""
        for command, task_uuid in commands_to_update.items():
            command.task_id = task_uuid

        self.update_model.objects.bulk_update(
            commands_to_update,
            ['task_id'],
        )

    def set_stage(self, command_id: int, stage_id: int) -> None:
        """Устанавливает ссылку на stage."""
        self.update_model.objects.filter(
            id=command_id,
        ).update(
            stage_id=stage_id,
        )


class BaseTaskStarter(ABC):
    """Запускает асинхронные задачи."""

    updater: BaseTaskProgressUpdater = None
    async_task: AsyncTask = None
    model_only_fields: Iterable[str] = ()

    def run(self, command_ids: Iterable[int], queue_level: Optional[int] = None) -> str:
        """Создает задачи и ставит их в очередь."""
        commands_to_update = {}
        skipped_commands_count = 0
        commands = self._get_commands(command_ids)
        queue_name = None

        if queue_level:
            queue_name = EntityLevelQueueTypeEnum.get_queue_name(level=queue_level)

        if not queue_name:
            queue_name = TASK_QUEUE_NAME

        for command in commands:
            if command.task_id:
                # Повторный запуск команды не допускается
                skipped_commands_count += 1
                continue

            try:
                async_result = self.async_task().apply_async(  # noqa pylint: disable=not-callable
                    args=None,
                    queue=queue_name,
                    routing_key=queue_name,
                    kwargs={
                        'command_id': command.id,
                    },
                    lock_data={
                        'lock_params': {
                            'command_id': f'{self.updater.update_model.__name__}_{command.id}',
                        },
                    },
                )
            except TaskUniqueException:
                skipped_commands_count += 1
                continue
            else:
                commands_to_update[command] = async_result.task_id

        if commands_to_update:
            self.updater().set_async_task(commands_to_update)  # noqa pylint: disable=not-callable

        message = f'Поставлено задач в очередь: {len(commands_to_update)} из {len(commands)}.'
        if skipped_commands_count:
            message += (
                f' Кол-во задач, которые были запущены ранее: {skipped_commands_count}. '
                'Однажды запущенные задачи не могут быть запущены снова!'
            )

        return message

    def _get_commands(self, command_ids: Iterable[int]) -> 'QuerySet':
        """Возвращает Queryset команд для создания задач."""
        return self.updater.update_model.objects.filter(
            id__in=command_ids,
        ).only(
            *self.model_only_fields,
        )


class Graph:
    """Граф связей между моделями.

    Предназначен для поиска кратчайшей связи между моделями и дальнейшего построения lookup`а
    до необходимого поля модели с последующим использованием его в фильтре.
    Вершинами графа выступают наименования моделей. Ребро содержит наименования модели связанной
    с другой моделью и наименования поля, через которое осуществляется связь.
    Вместо наименования поля может быть наименование обратной связи между моделями,
    в случае если данная связь является связью OneToOne.
    """

    def __init__(self):
        self.vertices: dict[str, dict[str, Optional[str]]] = {}
        """Словарь для хранения данных графа."""

    def add_vertex(self, vertex: str):
        """Добавление вершины."""
        if vertex not in self.vertices:
            self.vertices[vertex] = {}

    def add_edge(self, vertex1: str, vertex2: str, edge_name: str):
        """Добавление связи."""
        if vertex1 in self.vertices and vertex2 in self.vertices:
            self.vertices[vertex1][vertex2] = edge_name
            if vertex1 not in self.vertices[vertex2]:
                self.vertices[vertex2][vertex1] = None

    def remove_vertex(self, vertex: str):
        """Удаление вершины."""
        if vertex in self.vertices:
            del self.vertices[vertex]

            # Удаляем связанные с удаленной вершиной ребра
            for neighbour in self.vertices:
                if vertex in self.vertices[neighbour]:
                    self.vertices[neighbour].pop(vertex)

    def remove_edge(self, vertex1: str, vertex2: str):
        """Удаление связи."""
        if vertex1 in self.vertices and vertex2 in self.vertices:
            self.vertices[vertex1].pop(vertex2, None)
            self.vertices[vertex2].pop(vertex1, None)

    def get_vertices(self) -> list[str]:
        """Получение списка всех вершин."""
        return list(self.vertices)

    def get_edges(self) -> list[tuple[str, str, Optional[str]]]:
        """Получение всех связей."""
        edges = []

        for vertex, neighbors in self.vertices.items():
            for neighboring_vertex, edge_name in neighbors.items():
                edge = (vertex, neighboring_vertex, edge_name)
                edges.append(edge)

        return edges

    def __contains__(self, vertex: str) -> bool:
        return vertex in self.vertices

    def __iter__(self):
        return iter(self.vertices)

    def __getitem__(self, vertex: str):
        return self.vertices.get(vertex, {})

    def get_edges_between_vertices(
        self, from_vertex: str, to_vertex: str, required_edge_name: bool = True
    ) -> list[str]:
        """Получение списка наименований ребер между вершинами."""
        if from_vertex not in self.vertices and to_vertex not in self.vertices:
            return []

        path = []
        edge_weight = 1

        # Инициализация расстояния между вершинами
        distances = {vertex: float('inf') for vertex in self}
        distances[from_vertex] = 0

        priority_queue = [(0, from_vertex)]

        while priority_queue:
            current_distance, current_vertex = heapq.heappop(priority_queue)

            # Если достигнута конечная вершина, заканчиваем цикл
            if current_vertex == to_vertex:
                break

            # Проверяем все смежные вершины и обновляем расстояния, если находим более короткий путь
            for neighbor, edge_name in self[current_vertex].items():
                distance = current_distance + edge_weight
                if distance < distances[neighbor]:
                    distances[neighbor] = distance
                    heapq.heappush(priority_queue, (distance, neighbor))

        # Восстанавливаем путь от конечной вершины к начальной
        current_vertex = to_vertex
        while current_vertex != from_vertex:
            previous_vertex = edge_name = None

            for neighbor in self[current_vertex]:
                if distances[current_vertex] == distances[neighbor] + 1:
                    for neighbor_vertex, edge_name in self[neighbor].items():
                        if neighbor_vertex == current_vertex:
                            break

                    previous_vertex = neighbor
                    break

            if required_edge_name and not edge_name:
                path = []
                break

            path.append(edge_name)
            current_vertex = previous_vertex

        # Инвертируем путь и возвращаем его
        path.reverse()

        return path


def save_command_log_link(
    command: Union[RDMCollectingDataCommandProgress, RDMExportingDataCommandProgress], log_dir: str
) -> None:
    """Сохраняет ссылку на лог команды."""
    log_file = os.path.join(settings.MEDIA_ROOT, log_dir, f'{command.id}.log')
    if os.path.exists(log_file):
        command.logs_link = os.path.join(log_dir, f'{command.id}.log')
        command.save()


def make_download_link(fieldfile, text='Cкачать', show_filename=False):
    """Возвращает html ссылку для скачивания файла.

    Если show_filename == True, использует имя файла как текст ссылки
    """
    link = mark_safe('')
    if fieldfile:
        link_text = os.path.basename(fieldfile.name) if show_filename else text
        link = make_link(fieldfile.url, link_text)

    return link


def make_link(url, text):
    """Возвращает экаранированную html ссылку файла."""
    return format_html('<a href="{}" target="_blank" download>{}</a>', url, text)


def get_redis_version(connection: 'Redis') -> tuple[int, int, int]:
    """Возвращает кортеж с версией сервера Redis."""
    try:
        version = getattr(connection, '__redis_server_version', None)
        if not version:
            version = tuple([int(n) for n in connection.info('server')['redis_version'].split('.')[:3]])
            setattr(connection, '__redis_server_version', version)
    except ResponseError:
        version = (0, 0, 0)

    return version


def as_text(v: Union[bytes, str]) -> str:
    """Конвертирует последовательность байт в строку."""
    if isinstance(v, bytes):
        return v.decode('utf-8')
    elif isinstance(v, str):
        return v
    else:
        raise ValueError('Неизвестный тип %r' % type(v))



# Generated by Django 3.2.24 on 2025-06-28 14:10

import uuid

import django.db.models.deletion
import django.utils.timezone
from django.db import (
    migrations,
    models,
)

import educommon.django.db.mixins

import edu_rdm_integration.core.utils
import edu_rdm_integration.stages.export_data.helpers


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('edu_rdm_integration_entities', '0001_initial'),
        ('function_tools', '0005_auto_20220724_0050'),
        ('async_task', '0003_alter_runningtask_options'),
        ('rdm_collect_and_export_data', '0003_auto_20250704_0725'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name='ExportingDataStage',
                    fields=[
                        (
                            'id',
                            models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
                        ),
                        (
                            'period_started_at',
                            models.DateTimeField(
                                db_index=True, verbose_name='Левая граница периода выборки данных для выгрузки'
                            ),
                        ),
                        (
                            'period_ended_at',
                            models.DateTimeField(
                                db_index=True, verbose_name='Правая граница периода выборки данных для выгрузки'
                            ),
                        ),
                        (
                            'started_at',
                            models.DateTimeField(auto_now_add=True, verbose_name='Время начала выгрузки данных'),
                        ),
                        (
                            'ended_at',
                            models.DateTimeField(
                                blank=True, null=True, verbose_name='Время завершения выгрузки данных'
                            ),
                        ),
                        (
                            'manager',
                            models.ForeignKey(
                                blank=True,
                                null=True,
                                on_delete=django.db.models.deletion.PROTECT,
                                to='function_tools.entity',
                                verbose_name='Менеджер ранера Функции',
                            ),
                        ),
                    ],
                    options={
                        'verbose_name': 'Этап формирования данных для выгрузки',
                        'verbose_name_plural': 'Этапы формирования данных для выгрузки',
                        'db_table': 'rdm_exporting_data_stage',
                    },
                    bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
                ),
                migrations.CreateModel(
                    name='ExportingDataStageStatus',
                    fields=[
                        ('title', models.TextField(verbose_name='расшифровка значения')),
                        (
                            'key',
                            models.CharField(
                                db_index=True, max_length=512, primary_key=True, serialize=False, verbose_name='ключ'
                            ),
                        ),
                        ('order_number', models.PositiveIntegerField(default=100000, verbose_name='Порядковый номер')),
                    ],
                    options={
                        'verbose_name': 'Модель-перечисление статусов этапа выгрузки данных',
                        'verbose_name_plural': 'Модели-перечисления статусов этапов выгрузки данных',
                        'db_table': 'rdm_exporting_data_stage_status',
                    },
                ),
                migrations.CreateModel(
                    name='ExportingDataSubStage',
                    fields=[
                        (
                            'id',
                            models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
                        ),
                        (
                            'started_at',
                            models.DateTimeField(
                                auto_now_add=True, db_index=True, verbose_name='Время начала сбора данных'
                            ),
                        ),
                        (
                            'ended_at',
                            models.DateTimeField(
                                blank=True, db_index=True, null=True, verbose_name='Время завершения сбора данных'
                            ),
                        ),
                        (
                            'function',
                            models.ForeignKey(
                                blank=True,
                                null=True,
                                on_delete=django.db.models.deletion.PROTECT,
                                to='function_tools.entity',
                                verbose_name='Функция',
                            ),
                        ),
                        (
                            'stage',
                            models.ForeignKey(
                                on_delete=django.db.models.deletion.CASCADE,
                                to='edu_rdm_integration_export_data_stage.exportingdatastage',
                                verbose_name='Этап выгрузки данных',
                            ),
                        ),
                    ],
                    options={
                        'verbose_name': 'Стадия выгрузки данных',
                        'verbose_name_plural': 'Стадии выгрузки данных',
                        'db_table': 'rdm_exporting_data_sub_stage',
                    },
                    bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
                ),
                migrations.CreateModel(
                    name='ExportingDataSubStageStatus',
                    fields=[
                        ('title', models.TextField(verbose_name='расшифровка значения')),
                        (
                            'key',
                            models.CharField(
                                db_index=True, max_length=512, primary_key=True, serialize=False, verbose_name='ключ'
                            ),
                        ),
                        ('order_number', models.PositiveIntegerField(default=100000, verbose_name='Порядковый номер')),
                    ],
                    options={
                        'verbose_name': 'Модель-перечисление статусов подэтапа выгрузки данных',
                        'verbose_name_plural': 'Модели-перечисления статусов подэтапов выгрузки данных',
                        'db_table': 'rdm_exporting_data_sub_stage_status',
                    },
                ),
                migrations.CreateModel(
                    name='ExportingDataSubStageEntity',
                    fields=[
                        (
                            'id',
                            models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
                        ),
                        (
                            'entity',
                            models.ForeignKey(
                                on_delete=django.db.models.deletion.PROTECT,
                                to='edu_rdm_integration_entities.regionaldatamartentityenum',
                                verbose_name='Сущность РВД',
                            ),
                        ),
                        (
                            'exporting_data_sub_stage',
                            models.ForeignKey(
                                on_delete=django.db.models.deletion.CASCADE,
                                to='edu_rdm_integration_export_data_stage.exportingdatasubstage',
                                verbose_name='Подэтап выгрузки данных',
                            ),
                        ),
                    ],
                    options={
                        'verbose_name': 'Связь сущности и подэтапа выгрузки',
                        'verbose_name_plural': 'Связи сущности и подэтапа выгрузки',
                        'db_table': 'rdm_exporting_data_sub_stage_entity',
                    },
                ),
                migrations.CreateModel(
                    name='ExportingDataSubStageAttachment',
                    fields=[
                        (
                            'id',
                            models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
                        ),
                        (
                            'attachment',
                            models.FileField(
                                blank=True,
                                max_length=512,
                                null=True,
                                upload_to=edu_rdm_integration.stages.export_data.helpers.get_exporting_data_stage_attachment_path,
                                verbose_name='Сгенерированный файл',
                            ),
                        ),
                        (
                            'operation',
                            models.SmallIntegerField(
                                choices=[(1, 'Создание'), (2, 'Изменение'), (3, 'Удаление')], verbose_name='Действие'
                            ),
                        ),
                        ('created', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Дата создания')),
                        ('modified', models.DateTimeField(auto_now=True, null=True, verbose_name='Дата изменения')),
                        ('attachment_size', models.PositiveIntegerField(null=True, verbose_name='Размер файла (байт)')),
                        (
                            'exporting_data_sub_stage',
                            models.ForeignKey(
                                on_delete=django.db.models.deletion.CASCADE,
                                to='edu_rdm_integration_export_data_stage.exportingdatasubstage',
                                verbose_name='Подэтап выгрузки данных',
                            ),
                        ),
                    ],
                    options={
                        'verbose_name': 'Сгенерированный файл для дальнейшей выгрузки в "Региональная витрина данных"',
                        'verbose_name_plural': 'Сгенерированные файлы для дальнейшей выгрузки в "Региональная витрина данных"',
                        'db_table': 'rdm_exporting_data_sub_stage_attachment',
                    },
                    bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
                ),
                migrations.AddField(
                    model_name='exportingdatasubstage',
                    name='status',
                    field=models.ForeignKey(
                        default='CREATED',
                        on_delete=django.db.models.deletion.PROTECT,
                        to='edu_rdm_integration_export_data_stage.exportingdatasubstagestatus',
                        verbose_name='Статус',
                    ),
                ),
                migrations.AddField(
                    model_name='exportingdatastage',
                    name='status',
                    field=models.ForeignKey(
                        default='CREATED',
                        on_delete=django.db.models.deletion.PROTECT,
                        to='edu_rdm_integration_export_data_stage.exportingdatastagestatus',
                        verbose_name='Статус',
                    ),
                ),
                migrations.CreateModel(
                    name='EduRdmExportDataCommandProgress',
                    fields=[
                        (
                            'id',
                            models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
                        ),
                        (
                            'created',
                            models.DateTimeField(default=django.utils.timezone.now, verbose_name='Дата создания'),
                        ),
                        (
                            'period_started_at',
                            models.DateTimeField(verbose_name='Левая граница периода выборки данных для выгрузки'),
                        ),
                        (
                            'period_ended_at',
                            models.DateTimeField(verbose_name='Правая граница периода выборки данных для выгрузки'),
                        ),
                        ('generation_id', models.UUIDField(default=uuid.uuid4, verbose_name='Идентификатор генерации')),
                        (
                            'logs_link',
                            models.FileField(
                                max_length=255,
                                upload_to=edu_rdm_integration.core.utils.get_data_command_progress_attachment_path,
                                verbose_name='Файл лога',
                            ),
                        ),
                        (
                            'type',
                            models.PositiveSmallIntegerField(
                                choices=[(1, 'Автоматический'), (2, 'Ручной')], verbose_name='Тип команды'
                            ),
                        ),
                        (
                            'entity',
                            models.ForeignKey(
                                on_delete=django.db.models.deletion.PROTECT,
                                to='edu_rdm_integration_entities.regionaldatamartentityenum',
                                verbose_name='Сущность РВД',
                            ),
                        ),
                        (
                            'stage',
                            models.ForeignKey(
                                blank=True,
                                null=True,
                                on_delete=django.db.models.deletion.SET_NULL,
                                to='edu_rdm_integration_export_data_stage.exportingdatastage',
                                verbose_name='Этап выгрузки данных',
                            ),
                        ),
                        (
                            'task',
                            models.ForeignKey(
                                blank=True,
                                null=True,
                                on_delete=django.db.models.deletion.SET_NULL,
                                to='async_task.runningtask',
                                verbose_name='Асинхронная задача',
                            ),
                        ),
                    ],
                    options={
                        'verbose_name': 'Команда экспорта данных',
                        'verbose_name_plural': 'Команды экспорта данных',
                        'db_table': 'edu_rdm_exporting_data_command_progress',
                        'abstract': False,
                    },
                    bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
                ),
            ],
            database_operations=[],
        ),
    ]



from django.apps import (
    AppConfig,
)


class RDMTransferPipelineAppConfig(AppConfig):
    """Приложение для организации процесса сбора, экспорта в файлы и отправки файлов в витрину."""

    name = 'edu_rdm_integration.pipelines.transfer'
    label = 'edu_rdm_integration_transfer_pipeline'
    verbose_name = 'Приложение для организации процесса сбора, экспорта в файлы и отправки файлов в витрину.'

    def ready(self):
        """Готовность приложения."""
        if self.name == 'edu_rdm_integration.pipelines.transfer':
            # ruff: noqa: F401
            from . import (
                tasks,
            )



# Generated by Django 2.2.28 on 2023-07-13 17:32

import django.db.models.deletion
from django.db import (
    migrations,
    models,
)

import educommon.django.db.mixins
import educommon.utils.date

import edu_rdm_integration.stages.export_data.helpers


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('uploader_client', '0001_initial'),
        ('function_tools', '0005_auto_20220724_0050'),
    ]

    operations = [
        migrations.CreateModel(
            name='CollectingDataStageStatus',
            fields=[
                ('title', models.TextField(verbose_name='расшифровка значения')),
                (
                    'key',
                    models.CharField(
                        db_index=True, max_length=512, primary_key=True, serialize=False, verbose_name='ключ'
                    ),
                ),
                ('order_number', models.PositiveIntegerField(default=100000, verbose_name='Порядковый номер')),
            ],
            options={
                'verbose_name': 'Модель-перечисление статусов этапа сбора данных',
                'verbose_name_plural': 'Модели-перечисления статусов этапов сбора данных',
                'db_table': 'rdm_collecting_data_stage_status',
            },
        ),
        migrations.CreateModel(
            name='CollectingDataSubStageStatus',
            fields=[
                ('title', models.TextField(verbose_name='расшифровка значения')),
                (
                    'key',
                    models.CharField(
                        db_index=True, max_length=512, primary_key=True, serialize=False, verbose_name='ключ'
                    ),
                ),
                ('order_number', models.PositiveIntegerField(default=100000, verbose_name='Порядковый номер')),
            ],
            options={
                'verbose_name': 'Модель-перечисление статусов подэтапа сбора данных',
                'verbose_name_plural': 'Модели-перечисления статусов подэтапов сбора данных',
                'db_table': 'rdm_collecting_data_sub_stage_status',
            },
        ),
        migrations.CreateModel(
            name='CollectingExportedDataStage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'logs_period_started_at',
                    models.DateTimeField(
                        db_index=True,
                        default=educommon.utils.date.get_today_min_datetime,
                        verbose_name='Левая граница периода обрабатываемых логов',
                    ),
                ),
                (
                    'logs_period_ended_at',
                    models.DateTimeField(
                        db_index=True,
                        default=educommon.utils.date.get_today_max_datetime,
                        verbose_name='Правая граница периода обрабатываемых логов',
                    ),
                ),
                (
                    'started_at',
                    models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Время начала сбора данных'),
                ),
                (
                    'ended_at',
                    models.DateTimeField(
                        blank=True, db_index=True, null=True, verbose_name='Время завершения сбора данных'
                    ),
                ),
                (
                    'manager',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='function_tools.Entity',
                        verbose_name='Менеджер ранера Функции',
                    ),
                ),
                (
                    'status',
                    models.ForeignKey(
                        default='CREATED',
                        on_delete=django.db.models.deletion.PROTECT,
                        to='edu_rdm_integration.CollectingDataStageStatus',
                        verbose_name='Статус',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Этап формирования данных для выгрузки',
                'verbose_name_plural': 'Этапы формирования данных для выгрузки',
                'db_table': 'rdm_collecting_exported_data_stage',
            },
            bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ExportingDataStage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'period_started_at',
                    models.DateTimeField(
                        db_index=True, verbose_name='Левая граница периода выборки данных для выгрузки'
                    ),
                ),
                (
                    'period_ended_at',
                    models.DateTimeField(
                        db_index=True, verbose_name='Правая граница периода выборки данных для выгрузки'
                    ),
                ),
                ('started_at', models.DateTimeField(auto_now_add=True, verbose_name='Время начала выгрузки данных')),
                (
                    'ended_at',
                    models.DateTimeField(blank=True, null=True, verbose_name='Время завершения выгрузки данных'),
                ),
                (
                    'manager',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='function_tools.Entity',
                        verbose_name='Менеджер ранера Функции',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Этап формирования данных для выгрузки',
                'verbose_name_plural': 'Этапы формирования данных для выгрузки',
                'db_table': 'rdm_exporting_data_stage',
            },
            bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ExportingDataStageStatus',
            fields=[
                ('title', models.TextField(verbose_name='расшифровка значения')),
                (
                    'key',
                    models.CharField(
                        db_index=True, max_length=512, primary_key=True, serialize=False, verbose_name='ключ'
                    ),
                ),
                ('order_number', models.PositiveIntegerField(default=100000, verbose_name='Порядковый номер')),
            ],
            options={
                'verbose_name': 'Модель-перечисление статусов этапа выгрузки данных',
                'verbose_name_plural': 'Модели-перечисления статусов этапов выгрузки данных',
                'db_table': 'rdm_exporting_data_stage_status',
            },
        ),
        migrations.CreateModel(
            name='ExportingDataSubStage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'started_at',
                    models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Время начала сбора данных'),
                ),
                (
                    'ended_at',
                    models.DateTimeField(
                        blank=True, db_index=True, null=True, verbose_name='Время завершения сбора данных'
                    ),
                ),
                (
                    'function',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='function_tools.Entity',
                        verbose_name='Функция',
                    ),
                ),
                (
                    'stage',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='edu_rdm_integration.ExportingDataStage',
                        verbose_name='Этап выгрузки данных',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Стадия выгрузки данных',
                'verbose_name_plural': 'Стадии выгрузки данных',
                'db_table': 'rdm_exporting_data_sub_stage',
            },
            bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ExportingDataSubStageAttachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'attachment',
                    models.FileField(
                        blank=True,
                        max_length=512,
                        null=True,
                        upload_to=edu_rdm_integration.stages.export_data.helpers.get_exporting_data_stage_attachment_path,
                        verbose_name='Сгенерированный файл',
                    ),
                ),
                (
                    'operation',
                    models.SmallIntegerField(
                        choices=[(1, 'Создание'), (2, 'Изменение'), (3, 'Удаление')], verbose_name='Действие'
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Дата создания')),
                ('modified', models.DateTimeField(auto_now=True, null=True, verbose_name='Дата изменения')),
                (
                    'exporting_data_sub_stage',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='edu_rdm_integration.ExportingDataSubStage',
                        verbose_name='Подэтап выгрузки данных',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Сгенерированный файл для дальнейшей выгрузки в "Региональная витрина данных"',
                'verbose_name_plural': 'Сгенерированные файлы для дальнейшей выгрузки в "Региональная витрина данных"',
                'db_table': 'rdm_exporting_data_sub_stage_attachment',
            },
            bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ExportingDataSubStageStatus',
            fields=[
                ('title', models.TextField(verbose_name='расшифровка значения')),
                (
                    'key',
                    models.CharField(
                        db_index=True, max_length=512, primary_key=True, serialize=False, verbose_name='ключ'
                    ),
                ),
                ('order_number', models.PositiveIntegerField(default=100000, verbose_name='Порядковый номер')),
            ],
            options={
                'verbose_name': 'Модель-перечисление статусов подэтапа выгрузки данных',
                'verbose_name_plural': 'Модели-перечисления статусов подэтапов выгрузки данных',
                'db_table': 'rdm_exporting_data_sub_stage_status',
            },
        ),
        migrations.CreateModel(
            name='ExportingDataSubStageUploaderClientLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'request_id',
                    models.CharField(blank=True, max_length=100, verbose_name='Id запроса загрузки в витрину'),
                ),
                ('is_emulation', models.BooleanField(default=False, verbose_name='Включен режим эмуляции')),
                (
                    'file_upload_status',
                    models.SmallIntegerField(
                        blank=True,
                        choices=[
                            (1, 'В процессе загрузки в витрину'),
                            (2, 'Загрузка в витрину закончена'),
                            (3, 'Ошибка загрузки в витрину'),
                        ],
                        null=True,
                        verbose_name='Общий статус загрузки файла в витрину',
                    ),
                ),
                (
                    'created',
                    models.DateTimeField(auto_now_add=True, db_index=True, null=True, verbose_name='Дата создания'),
                ),
                (
                    'modified',
                    models.DateTimeField(auto_now=True, db_index=True, null=True, verbose_name='Дата изменения'),
                ),
                (
                    'attachment',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='edu_rdm_integration.ExportingDataSubStageAttachment',
                        verbose_name='Прикрепленный файл',
                    ),
                ),
                (
                    'entry',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='uploader_client_log',
                        to='uploader_client.Entry',
                        verbose_name='Лог запроса и ответа',
                    ),
                ),
                (
                    'sub_stage',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='edu_rdm_integration.ExportingDataSubStage',
                        verbose_name='Подэтап выгрузки данных',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Лог запроса подэтапа выгрузки данных',
                'verbose_name_plural': 'Лог запроса подэтапа выгрузки данных',
                'db_table': 'rdm_exporting_data_sub_stage_uploader_client_log',
            },
            bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='RegionalDataMartEntityEnum',
            fields=[
                ('title', models.TextField(verbose_name='расшифровка значения')),
                (
                    'key',
                    models.CharField(
                        db_index=True, max_length=512, primary_key=True, serialize=False, verbose_name='ключ'
                    ),
                ),
                ('order_number', models.PositiveIntegerField(default=100000, verbose_name='Порядковый номер')),
            ],
            options={
                'verbose_name': 'Модель-перечисление сущностей "Региональной витрины данных"',
                'verbose_name_plural': 'Модели-перечисления сущностей "Региональной витрины данных"',
                'db_table': 'rdm_entity',
            },
        ),
        migrations.CreateModel(
            name='RegionalDataMartModelEnum',
            fields=[
                ('title', models.TextField(verbose_name='расшифровка значения')),
                (
                    'key',
                    models.CharField(
                        db_index=True, max_length=512, primary_key=True, serialize=False, verbose_name='ключ'
                    ),
                ),
                ('order_number', models.PositiveIntegerField(default=100000, verbose_name='Порядковый номер')),
            ],
            options={
                'verbose_name': 'Модель-перечисление моделей "Региональной витрины данных"',
                'verbose_name_plural': 'Модели-перечисления моделей "Региональной витрины данных"',
                'db_table': 'rdm_model',
            },
        ),
        migrations.CreateModel(
            name='UploadStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'code',
                    models.SmallIntegerField(blank=True, null=True, verbose_name='Код статуса загрузки в витрину'),
                ),
                ('description', models.CharField(max_length=100, verbose_name='Описание статуса загрузки в витрину')),
            ],
            options={
                'verbose_name': 'Статус загрузки данных в витрину',
                'verbose_name_plural': 'Статусы загрузки данных в витрину',
                'db_table': 'rdm_upload_status',
            },
            bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='UploadStatusRequestLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'entry',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='upload_status_request_log',
                        to='uploader_client.Entry',
                        verbose_name='Cвязь запроса статуса с запросом в витрину',
                    ),
                ),
                (
                    'status',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='edu_rdm_integration.UploadStatus',
                        verbose_name='Cвязь с таблицей возможных статусов',
                    ),
                ),
                (
                    'upload',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to='edu_rdm_integration.ExportingDataSubStageUploaderClientLog',
                        verbose_name='Cвязь запроса статуса с загрузкой файла в витрину',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Лог запроса подэтапа выгрузки данных',
                'verbose_name_plural': 'Логи запроса подэтапа выгрузки данных',
                'db_table': 'rdm_upload_status_request_log',
            },
            bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
        ),
        migrations.AddField(
            model_name='exportingdatasubstage',
            name='status',
            field=models.ForeignKey(
                default='CREATED',
                on_delete=django.db.models.deletion.PROTECT,
                to='edu_rdm_integration.ExportingDataSubStageStatus',
                verbose_name='Статус',
            ),
        ),
        migrations.AddField(
            model_name='exportingdatastage',
            name='status',
            field=models.ForeignKey(
                default='CREATED',
                on_delete=django.db.models.deletion.PROTECT,
                to='edu_rdm_integration.ExportingDataStageStatus',
                verbose_name='Статус',
            ),
        ),
        migrations.CreateModel(
            name='CollectingExportedDataSubStage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'started_at',
                    models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Время начала сбора данных'),
                ),
                (
                    'ended_at',
                    models.DateTimeField(
                        blank=True, db_index=True, null=True, verbose_name='Время завершения сбора данных'
                    ),
                ),
                (
                    'function',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT, to='function_tools.Entity', verbose_name='Функция'
                    ),
                ),
                (
                    'previous',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to='edu_rdm_integration.CollectingExportedDataSubStage',
                        verbose_name='Предыдущий сбор данных',
                    ),
                ),
                (
                    'stage',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to='edu_rdm_integration.CollectingExportedDataStage',
                        verbose_name='Этап подготовки данных для экспорта',
                    ),
                ),
                (
                    'status',
                    models.ForeignKey(
                        default='CREATED',
                        on_delete=django.db.models.deletion.PROTECT,
                        to='edu_rdm_integration.CollectingDataSubStageStatus',
                        verbose_name='Статус',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Подэтап формирования данных для выгрузки',
                'verbose_name_plural': 'Подэтапы формирования данных для выгрузки',
                'db_table': 'rdm_collecting_exported_data_sub_stage',
            },
            bases=(educommon.django.db.mixins.ReprStrPreModelMixin, models.Model),
        ),
    ]



from datetime import (
    datetime,
)
from typing import (
    TYPE_CHECKING,
    Optional,
)

import celery
from celery.schedules import (
    crontab,
)
from django.conf import (
    settings,
)
from django.utils import (
    timezone,
)

from educommon import (
    logger,
)
from educommon.async_task.models import (
    AsyncTaskType,
    RunningTask,
)
from educommon.async_task.tasks import (
    UniquePeriodicAsyncTask,
)
from educommon.django.db.mixins.validation import (
    QuerySet,
)
from educommon.utils.date import (
    get_today_min_datetime,
)

from edu_rdm_integration.core.consts import (
    FAST_TRANSFER_TASK_QUEUE_NAME,
    LONG_TRANSFER_TASK_QUEUE_NAME,
    TASK_QUEUE_NAME,
)
from edu_rdm_integration.core.enums import (
    CommandType,
)
from edu_rdm_integration.core.helpers import (
    save_command_log_link,
)
from edu_rdm_integration.pipelines.transfer.enums import (
    EntityLevelQueueTypeEnum,
)
from edu_rdm_integration.pipelines.transfer.mixins import (
    BaseTransferLatestEntitiesDataMixin,
)
from edu_rdm_integration.pipelines.transfer.models import (
    TransferredEntity,
)
from edu_rdm_integration.stages.collect_data.models import (
    RDMCollectingDataCommandProgress,
)
from edu_rdm_integration.stages.collect_data.operations import (
    BaseCollectLatestModelsData,
)
from edu_rdm_integration.stages.export_data.models import (
    RDMExportingDataCommandProgress,
)
from edu_rdm_integration.stages.export_data.operations import (
    ExportLatestEntitiesData,
)


if TYPE_CHECKING:
    from django.db.models import (
        QuerySet,
    )


class BaseTransferLatestEntitiesDataPeriodicTask(BaseTransferLatestEntitiesDataMixin, UniquePeriodicAsyncTask):
    """Базовая периодическая задача сбора и выгрузки данных для переиспользования в разных очередях."""

    def __init__(self) -> None:
        super().__init__()

        self._period_ended_at: Optional[datetime] = None

    def _get_period_ended_at(self):
        """Определяет единую дату окончания периода сбора.

        Это нужно для исключения разного времени окончания сбора при явном запуске функции - чтобы
        избежать ошибки экспорта при недосборе данных.
        """
        if not self._period_ended_at:
            self._period_ended_at = timezone.now()

        return self._period_ended_at

    def _run_collect_model_data(self, model: str, task_id: str) -> None:
        """Запускает сбор данных модели РВД."""
        command = self._create_collect_command(model, task_id)
        collect_model_data = self._prepare_collect_model_data_class(command)
        collect_model_data.collect()

        command.refresh_from_db(fields=['stage_id'])
        save_command_log_link(command, settings.RDM_COLLECT_LOG_DIR)

    def _run_export_entity_data(self, entity: str, task_id: str) -> None:
        """Запускает экспорт данных сущности РВД."""
        command = self._create_export_command(entity, task_id)
        if command:
            export_entity_data = self._prepare_export_entity_data_class(command)
            export_entity_data.export()

            command.refresh_from_db(fields=['stage_id'])
            save_command_log_link(command, settings.RDM_EXPORT_LOG_DIR)

    def _create_collect_command(self, model: str, task_id: str) -> RDMCollectingDataCommandProgress:
        """Создает команду сбора данных моделей РВД."""
        manager = self._collecting_data_managers[model]
        manager_last_collected = (
            self._collecting_data_manager_to_logs_period_end.get(manager.uuid) or get_today_min_datetime()
        )

        period_started_at = manager_last_collected
        period_ended_at = self._get_period_ended_at()

        return RDMCollectingDataCommandProgress.objects.create(
            model_id=model,
            logs_period_started_at=period_started_at,
            logs_period_ended_at=period_ended_at,
            task_id=task_id,
            type=CommandType.AUTO,
        )

    def _create_export_command(self, entity: str, task_id: str) -> Optional[RDMExportingDataCommandProgress]:
        """Создает команду экспорта данных сущностей РВД."""
        manager = self._exporting_data_managers[entity]
        manager_last_exported = self._exporting_data_manager_to_period_end.get(manager.uuid)

        if manager_last_exported:
            period_started_at = manager_last_exported
            period_ended_at = timezone.now()

            return RDMExportingDataCommandProgress.objects.create(
                entity_id=entity,
                period_started_at=period_started_at,
                period_ended_at=period_ended_at,
                task_id=task_id,
                type=CommandType.AUTO,
            )

        return None

    def _prepare_collect_model_data_class(
        self, command: RDMCollectingDataCommandProgress
    ) -> BaseCollectLatestModelsData:
        """Подготавливает объект класса сбора данных моделей РВД."""
        return BaseCollectLatestModelsData(
            models=[command.model_id],
            logs_period_started_at=command.logs_period_started_at,
            logs_period_ended_at=command.logs_period_ended_at,
            command_id=command.id,
            use_times_limit=True,
        )

    def _prepare_export_entity_data_class(self, command: RDMExportingDataCommandProgress) -> ExportLatestEntitiesData:
        """Подготавливает объект класса экспорта данных сущностей РВД.

        При экспорте данных передаем параметр task_id для обновления поля "Описание"
        наименованиями выгруженных сущностей.
        """
        return ExportLatestEntitiesData(
            entities=[command.entity_id],
            period_started_at=command.period_started_at,
            period_ended_at=command.period_ended_at,
            command_id=command.id,
            task_id=self.request.id,
        )

    def process(self, *args, **kwargs):
        """Выполняет задачу."""
        super().process(*args, **kwargs)

        self._period_ended_at = None
        self.prepare_collect_export_managers()

        task_id = (
            RunningTask.objects.filter(
                pk=self.request.id,
            )
            .values_list('pk', flat=True)
            .first()
        )

        collected_entity_models = set()

        for entity_enum, export_enabled in sorted(
            self._transferred_entities, key=lambda entity: entity[0].order_number
        ):
            entity_models = self._entites_models_map.get(entity_enum.key, ())
            for model_enum_value in entity_models:
                if model_enum_value.key not in collected_entity_models:
                    collected_entity_models.add(model_enum_value.key)
                    try:
                        self._run_collect_model_data(model_enum_value.key, task_id)
                    except Exception as e:
                        logger.warning(e)

                        continue

            try:
                if export_enabled:
                    self._run_export_entity_data(entity_enum.key, task_id)
            except Exception as e:
                logger.warning(e)

                continue


class TransferLatestEntitiesDataPeriodicTask(BaseTransferLatestEntitiesDataPeriodicTask):
    """Периодическая задача сбора и выгрузки данных."""

    queue = TASK_QUEUE_NAME
    routing_key = TASK_QUEUE_NAME
    description = 'Периодическая задача сбора и экспорта данных РВД'
    lock_expire_seconds = settings.RDM_TRANSFER_TASK_LOCK_EXPIRE_SECONDS
    task_type = AsyncTaskType.UNKNOWN
    run_every = crontab(
        minute=settings.RDM_TRANSFER_TASK_MINUTE,
        hour=settings.RDM_TRANSFER_TASK_HOUR,
        day_of_week=settings.RDM_TRANSFER_TASK_DAY_OF_WEEK,
    )

    def get_entity_qs(self) -> 'QuerySet[TransferredEntity]':
        """Возвращает QuerySet сущностей сбора и выгрузки."""
        return TransferredEntity.objects.filter(queue_level=EntityLevelQueueTypeEnum.BASE)


class TransferLatestEntitiesDataFastPeriodicTask(BaseTransferLatestEntitiesDataPeriodicTask):
    """Периодическая задача сбора и выгрузки данных для быстрого уровня очереди."""

    queue = FAST_TRANSFER_TASK_QUEUE_NAME
    routing_key = FAST_TRANSFER_TASK_QUEUE_NAME
    description = 'Периодическая задача сбора и экспорта данных РВД (быстрый уровень)'
    lock_expire_seconds = settings.RDM_FAST_TRANSFER_TASK_LOCK_EXPIRE_SECONDS
    task_type = AsyncTaskType.UNKNOWN
    run_every = crontab(
        minute=settings.RDM_FAST_TRANSFER_TASK_MINUTE,
        hour=settings.RDM_FAST_TRANSFER_TASK_HOUR,
        day_of_week=settings.RDM_FAST_TRANSFER_TASK_DAY_OF_WEEK,
    )

    def get_entity_qs(self) -> 'QuerySet[TransferredEntity]':
        """Возвращает QuerySet сущностей сбора и выгрузки."""
        return TransferredEntity.objects.filter(queue_level=EntityLevelQueueTypeEnum.FAST)


class TransferLatestEntitiesDataLongPeriodicTask(BaseTransferLatestEntitiesDataPeriodicTask):
    """Периодическая задача сбора и выгрузки данных для долгого уровня очереди."""

    queue = LONG_TRANSFER_TASK_QUEUE_NAME
    routing_key = LONG_TRANSFER_TASK_QUEUE_NAME
    description = 'Периодическая задача сбора и экспорта данных РВД (долгий уровень)'
    lock_expire_seconds = settings.RDM_LONG_TRANSFER_TASK_LOCK_EXPIRE_SECONDS
    task_type = AsyncTaskType.UNKNOWN
    run_every = crontab(
        minute=settings.RDM_LONG_TRANSFER_TASK_MINUTE,
        hour=settings.RDM_LONG_TRANSFER_TASK_HOUR,
        day_of_week=settings.RDM_LONG_TRANSFER_TASK_DAY_OF_WEEK,
    )

    def get_entity_qs(self) -> 'QuerySet[TransferredEntity]':
        """Возвращает QuerySet сущностей сбора и выгрузки."""
        return TransferredEntity.objects.filter(queue_level=EntityLevelQueueTypeEnum.LONG)


celery_app = celery.app.app_or_default()
celery_app.register_task(TransferLatestEntitiesDataPeriodicTask)
celery_app.register_task(TransferLatestEntitiesDataFastPeriodicTask)
celery_app.register_task(TransferLatestEntitiesDataLongPeriodicTask)



import importlib
from typing import (
    TYPE_CHECKING,
    Optional,
)

from edu_rdm_integration.rdm_models.models import (
    RDMModelEnum,
)


if TYPE_CHECKING:
    from m3_db_utils.models import (
        ModelEnumValue,
    )


class ModelOutdatedDataCleanerManager:
    """Управляет очисткой устаревших данных моделей РВД.

    Получает все модели РВД зарегистрированные в модели-перечислении edu_rdm_integration.rdm_models.models.RDMModelEnum.
    У каждой модели проверяет наличие поля outdated_data_cleaners. Если такое поле есть, то инстанцирует его и вызывает
    метод run.
    """

    def __init__(
        self,
        *args,
        models: Optional[list[str]] = None,
        safe: bool = False,
        log_sql: bool = False,
        **kwargs,
    ):
        self._models = models
        self._safe = safe
        self._log_sql = log_sql

        super().__init__(*args, **kwargs)

    def _process_model(self, model_enum_value: 'ModelEnumValue'):
        """Обрабатывает модель РВД."""
        if not hasattr(model_enum_value, 'outdated_data_cleaners'):
            return

        for outdated_data_cleaner in model_enum_value.outdated_data_cleaners:
            if isinstance(outdated_data_cleaner, str):
                outdated_data_cleaner_module_path, cleaner_name = outdated_data_cleaner.rsplit('.', 1)
                outdated_data_cleaner_module = importlib.import_module(outdated_data_cleaner_module_path)
                outdated_data_cleaner = getattr(outdated_data_cleaner_module, cleaner_name)

            outdated_data_cleaner(model_enum_value=model_enum_value, safe=self._safe, log_sql=self._log_sql).run()

    def _process_models(self):
        """Обрабатывает все модели РВД."""
        for model_enum_value in RDMModelEnum.get_model_enum_values():
            if self._models is not None and model_enum_value.key not in self._models:
                continue

            self._process_model(model_enum_value=model_enum_value)

    def run(self):
        """Запускает очистку устаревших данных моделей РВД."""
        self._process_models()



# Generated by Django 3.2.24 on 2025-07-04 08:10

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('function_tools', '0005_auto_20220724_0050'),
        ('edu_rdm_integration_collect_data_stage', '0002_edurdmcollectdatacommandprogress'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='CollectingDataStageStatus',
            new_name='RDMCollectingDataStageStatus',
        ),
        migrations.RenameModel(
            old_name='CollectingExportedDataStage',
            new_name='RDMCollectingDataStage',
        ),
        migrations.RenameModel(
            old_name='CollectingDataSubStageStatus',
            new_name='RDMCollectingDataSubStageStatus',
        ),
        migrations.RenameModel(
            old_name='CollectingExportedDataSubStage',
            new_name='RDMCollectingDataSubStage',
        ),
        migrations.RenameModel(
            old_name='EduRdmCollectDataCommandProgress',
            new_name='RDMCollectingDataCommandProgress',
        ),
    ]



# Generated by Django 3.2.24 on 2025-07-04 08:10

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('function_tools', '0005_auto_20220724_0050'),
        ('edu_rdm_integration_upload_data_stage', '0001_initial'),
        ('async_task', '0003_alter_runningtask_options'),
        ('edu_rdm_integration_entities', '0002_rename_regionaldatamartentityenum_rdmentityenum'),
        ('edu_rdm_integration_export_data_stage', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='EduRdmExportDataCommandProgress',
            new_name='RDMExportingDataCommandProgress',
        ),
        migrations.RenameModel(
            old_name='ExportingDataStage',
            new_name='RDMExportingDataStage',
        ),
        migrations.RenameModel(
            old_name='ExportingDataStageStatus',
            new_name='RDMExportingDataStageStatus',
        ),
        migrations.RenameModel(
            old_name='ExportingDataSubStage',
            new_name='RDMExportingDataSubStage',
        ),
        migrations.RenameModel(
            old_name='ExportingDataSubStageAttachment',
            new_name='RDMExportingDataSubStageAttachment',
        ),
        migrations.RenameModel(
            old_name='ExportingDataSubStageEntity',
            new_name='RDMExportingDataSubStageEntity',
        ),
        migrations.RenameModel(
            old_name='ExportingDataSubStageStatus',
            new_name='RDMExportingDataSubStageStatus',
        ),
    ]



from typing import (
    Optional,
)

from edu_rdm_integration.stages.service.service_outdated_data.cleaners.collect_data import (
    CollectingDataCommandProgressCleaner,
    CollectingDataStageCleaner,
    CollectingDataSubStageCleaner,
)
from edu_rdm_integration.stages.service.service_outdated_data.cleaners.export_data import (
    ExportingDataCommandProgressCleaner,
    ExportingDataStageCleaner,
    ExportingDataSubStageAttachmentCleaner,
    ExportingDataSubStageCleaner,
    ExportingDataSubStageEntityCleaner,
)
from edu_rdm_integration.stages.service.service_outdated_data.cleaners.upload_data import (
    EntryCleaner,
    ExportingDataSubStageUploaderClientLogCleaner,
    UploadStatusRequestLogCleaner,
)
from educommon import (
    logger,
)



class ServiceOutdatedDataCleanerManager:
    """Управляет очисткой устаревших сервисных данных по разным этапам."""

    STAGES_MAPPING = {
        'collect': [
            CollectingDataSubStageCleaner,
            CollectingDataStageCleaner,
            CollectingDataCommandProgressCleaner,
        ],
        'export': [
            ExportingDataSubStageCleaner,
            ExportingDataStageCleaner,
            ExportingDataSubStageAttachmentCleaner,
            ExportingDataSubStageEntityCleaner,
            ExportingDataCommandProgressCleaner,
        ],
        'upload': [
            ExportingDataSubStageUploaderClientLogCleaner,
            UploadStatusRequestLogCleaner,
            EntryCleaner,
        ],
    }

    def __init__(
        self,
        *args,
        stages: Optional[list[str]]= None,
        safe: bool = False,
        log_sql: bool = False,
        **kwargs
    ):
        """Инициализация менеджера."""
        self._stages = stages
        self._safe = safe
        self._log_sql = log_sql

        super().__init__(*args, **kwargs)

    def _process_stage(self, stage: str):
        """Запускает все зарегистрированные уборщики для указанного этапа."""
        cleaners = self.STAGES_MAPPING.get(stage, [])
        if not cleaners:
            logger.info(f'Для этапа "{stage}" нет зарегистрированных уборщиков.')
            return

        for cleaner_cls in cleaners:
            cleaner_cls(safe=self._safe, log_sql=self._log_sql).run()

    def run(self):
        """Запускает очистку устаревших данных сервисных моделей РВД."""
        stages_to_process = self._stages or self.STAGES_MAPPING.keys()
        for stage in stages_to_process:
            self._process_stage(stage)
